@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBufferSoft.cxx 2488 2012-05-20 18:26:59Z stephena $
+// $Id: FrameBufferSoft.cxx 2579 2013-01-04 19:49:01Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -57,7 +57,7 @@ bool FrameBufferSoft::initSubsystem(VideoMode& mode)
 
   if(!myRectList)
   {
-    myOSystem->logMessage("ERROR: Unable to get memory for SDL rects\n", 0);
+    myOSystem->logMessage("ERROR: Unable to get memory for SDL rects", 0);
     return false;
   }
 
@@ -802,13 +802,13 @@ void FBSurfaceSoft::setPos(uInt32 x, uInt32 y)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSoft::setWidth(uInt32 w)
 {
-  myWidth = w;
+  myWidth = BSPF_min(w, (uInt32)mySurface->w);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FBSurfaceSoft::setHeight(uInt32 h)
 {
-  myHeight = h;
+  myHeight = BSPF_min(h, (uInt32)mySurface->h);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
