@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundSDL.cxx 2499 2012-05-25 12:41:19Z stephena $
+// $Id: SoundSDL.cxx 2726 2013-05-08 23:34:42Z stephena $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
@@ -43,7 +43,7 @@ SoundSDL::SoundSDL(OSystem* osystem)
     myIsMuted(true),
     myVolume(100)
 {
-  myOSystem->logMessage("SoundSDL::SoundSDL started ...\n", 2);
+  myOSystem->logMessage("SoundSDL::SoundSDL started ...", 2);
 
   // The sound system is opened only once per program run, to eliminate
   // issues with opening and closing it multiple times
@@ -51,7 +51,7 @@ SoundSDL::SoundSDL(OSystem* osystem)
   // whereby sound stopped working after the first video change
   SDL_AudioSpec desired;
   desired.freq   = myOSystem->settings().getInt("freq");
-  desired.format = AUDIO_U8;
+  desired.format = AUDIO_S16SYS;
   desired.channels = 2;
   desired.samples  = myOSystem->settings().getInt("fragsize");
   desired.callback = callback;
@@ -87,7 +87,7 @@ SoundSDL::SoundSDL(OSystem* osystem)
   myIsInitializedFlag = true;
   SDL_PauseAudio(1);
 
-  myOSystem->logMessage("SoundSDL::SoundSDL initialized\n", 2);
+  myOSystem->logMessage("SoundSDL::SoundSDL initialized", 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -100,40 +100,35 @@ SoundSDL::~SoundSDL()
     myIsEnabled = myIsInitializedFlag = false;
   }
 
-  myOSystem->logMessage("SoundSDL destroyed\n", 2);
+  myOSystem->logMessage("SoundSDL destroyed", 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SoundSDL::setEnabled(bool state)
 {
-  myOSystem->settings().setBool("sound", state);
+  myOSystem->settings().setValue("sound", state);
 
-  myOSystem->logMessage(state ? "SoundSDL::setEnabled(true)\n" : 
-                                "SoundSDL::setEnabled(false)\n", 2);
+  myOSystem->logMessage(state ? "SoundSDL::setEnabled(true)" : 
+                                "SoundSDL::setEnabled(false)", 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SoundSDL::open()
 {
-  myOSystem->logMessage("SoundSDL::open started ...\n", 2);
+  myOSystem->logMessage("SoundSDL::open started ...", 2);
   myIsEnabled = false;
   mute(true);
   if(!myIsInitializedFlag || !myOSystem->settings().getBool("sound"))
   {
-    myOSystem->logMessage("Sound disabled\n\n", 1);
+    myOSystem->logMessage("Sound disabled\n", 1);
     return;
   }
 
   // Now initialize the TIASound object which will actually generate sound
-  int tiafreq = myOSystem->settings().getInt("tiafreq");
   myTIASound.outputFrequency(myHardwareSpec.freq);
-  myTIASound.tiaFrequency(tiafreq);
   const string& chanResult =
       myTIASound.channels(myHardwareSpec.channels, myNumChannels == 2);
 
-  bool clipvol = myOSystem->settings().getBool("clipvol");
-  myTIASound.clipVolume(clipvol);
-
   // Adjust volume to that defined in settings
   myVolume = myOSystem->settings().getInt("volume");
   setVolume(myVolume);
@@ -144,11 +139,8 @@ void SoundSDL::open()
       << "  Volume:      " << (int)myVolume << endl
       << "  Frag size:   " << (int)myHardwareSpec.samples << endl
       << "  Frequency:   " << (int)myHardwareSpec.freq << endl
-      << "  Format:      " << (int)myHardwareSpec.format << endl
-      << "  TIA Freq:    " << (int)tiafreq << endl
       << "  Channels:    " << (int)myHardwareSpec.channels
                            << " (" << chanResult << ")" << endl
-      << "  Clip volume: " << (clipvol ? "on" : "off") << endl
       << endl;
   myOSystem->logMessage(buf.str(), 1);
 
@@ -156,7 +148,7 @@ void SoundSDL::open()
   myIsEnabled = true;
   mute(false);
 
-  myOSystem->logMessage("SoundSDL::open finished\n", 2);
+  myOSystem->logMessage("SoundSDL::open finished", 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -169,7 +161,7 @@ void SoundSDL::close()
     myLastRegisterSetCycle = 0;
     myTIASound.reset();
     myRegWriteQueue.clear();
-    myOSystem->logMessage("SoundSDL::close\n", 2);
+    myOSystem->logMessage("SoundSDL::close", 2);
   }
 }
 
@@ -201,7 +193,7 @@ void SoundSDL::setVolume(Int32 percent)
 {
   if(myIsInitializedFlag && (percent >= 0) && (percent <= 100))
   {
-    myOSystem->settings().setInt("volume", percent);
+    myOSystem->settings().setValue("volume", percent);
     SDL_LockAudio();
     myVolume = percent;
     myTIASound.volume(percent);
@@ -262,7 +254,7 @@ void SoundSDL::set(uInt16 addr, uInt8 value, Int32 cycle)
 {
   SDL_LockAudio();
 
-  // First, calulate how many seconds would have past since the last
+  // First, calculate how many seconds would have past since the last
   // register write on a real 2600
   double delta = (((double)(cycle - myLastRegisterSetCycle)) / 
       (1193191.66666667));
@@ -284,7 +276,7 @@ void SoundSDL::set(uInt16 addr, uInt8 value, Int32 cycle)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL::processFragment(uInt8* stream, Int32 length)
+void SoundSDL::processFragment(Int16* stream, uInt32 length)
 {
   uInt32 channels = myHardwareSpec.channels;
   length = length / channels;
@@ -311,7 +303,6 @@ void SoundSDL::processFragment(uInt8* stream, Int32 length)
     {
       // There are no more pending TIA sound register updates so we'll
       // use the current settings to finish filling the sound fragment
-//    myTIASound.process(stream + (uInt32)position, length - (uInt32)position);
       myTIASound.process(stream + ((uInt32)position * channels),
           length - (uInt32)position);
 
@@ -341,9 +332,6 @@ void SoundSDL::processFragment(uInt8* stream, Int32 length)
           // Process the fragment upto the next TIA register write.  We
           // round the count passed to process up if needed.
           double samples = (myHardwareSpec.freq * info.delta);
-//        myTIASound.process(stream + (uInt32)position, (uInt32)samples +
-//            (uInt32)(position + samples) - 
-//            ((uInt32)position + (uInt32)samples));
           myTIASound.process(stream + ((uInt32)position * channels),
               (uInt32)samples + (uInt32)(position + samples) - 
               ((uInt32)position + (uInt32)samples));
@@ -359,7 +347,6 @@ void SoundSDL::processFragment(uInt8* stream, Int32 length)
         // The next register update occurs in the next fragment so finish
         // this fragment with the current TIA settings and reduce the register
         // update delay by the corresponding amount of time
-//      myTIASound.process(stream + (uInt32)position, length - (uInt32)position);
         myTIASound.process(stream + ((uInt32)position * channels),
             length - (uInt32)position);
         info.delta -= duration;
@@ -374,7 +361,12 @@ void SoundSDL::callback(void* udata, uInt8* stream, int len)
 {
   SoundSDL* sound = (SoundSDL*)udata;
   if(sound->myIsEnabled)
-    sound->processFragment(stream, (Int32)len);
+  {
+    // The callback is requesting 8-bit (unsigned) data, but the TIA sound
+    // emulator deals in 16-bit (signed) data
+    // So, we need to convert the pointer and half the length
+    sound->processFragment((Int16*)stream, (uInt32)len / 2);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
