@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: NTSCFilter.cxx 2501 2012-05-26 22:18:13Z stephena $
+// $Id: NTSCFilter.cxx 2726 2013-05-08 23:34:42Z stephena $
 //============================================================================
 
 #include "FrameBuffer.hxx"
@@ -110,6 +110,20 @@ string NTSCFilter::setPreset(Preset preset)
   return msg;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string NTSCFilter::getPreset() const
+{
+  switch(myPreset)
+  {
+    case PRESET_COMPOSITE:  return "COMPOSITE";
+    case PRESET_SVIDEO:     return "S-VIDEO";
+    case PRESET_RGB:        return "RGB";
+    case PRESET_BAD:        return "BAD ADJUST";
+    case PRESET_CUSTOM:     return "CUSTOM";
+    default:                return "Disabled";
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string NTSCFilter::setNextAdjustable()
 {
@@ -196,16 +210,16 @@ void NTSCFilter::loadConfig(const Settings& settings)
 void NTSCFilter::saveConfig(Settings& settings) const
 {
   // Save adjustables for custom mode
-  settings.setFloat("tv_hue", myCustomSetup.hue);
-  settings.setFloat("tv_saturation", myCustomSetup.saturation);
-  settings.setFloat("tv_contrast", myCustomSetup.contrast);
-  settings.setFloat("tv_brightness", myCustomSetup.brightness);
-  settings.setFloat("tv_sharpness", myCustomSetup.sharpness);
-  settings.setFloat("tv_gamma", myCustomSetup.gamma);
-  settings.setFloat("tv_resolution", myCustomSetup.resolution);
-  settings.setFloat("tv_artifacts", myCustomSetup.artifacts);
-  settings.setFloat("tv_fringing", myCustomSetup.fringing);
-  settings.setFloat("tv_bleed", myCustomSetup.bleed);
+  settings.setValue("tv_hue", myCustomSetup.hue);
+  settings.setValue("tv_saturation", myCustomSetup.saturation);
+  settings.setValue("tv_contrast", myCustomSetup.contrast);
+  settings.setValue("tv_brightness", myCustomSetup.brightness);
+  settings.setValue("tv_sharpness", myCustomSetup.sharpness);
+  settings.setValue("tv_gamma", myCustomSetup.gamma);
+  settings.setValue("tv_resolution", myCustomSetup.resolution);
+  settings.setValue("tv_artifacts", myCustomSetup.artifacts);
+  settings.setValue("tv_fringing", myCustomSetup.fringing);
+  settings.setValue("tv_bleed", myCustomSetup.bleed);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
