@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Debugger.cxx 2359 2012-01-17 22:20:20Z stephena $
+// $Id: Debugger.cxx 2753 2013-06-21 12:15:32Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -40,6 +40,7 @@
 #include "TIA.hxx"
 
 #include "CartDebug.hxx"
+#include "CartDebugWidget.hxx"
 #include "CpuDebug.hxx"
 #include "RiotDebug.hxx"
 #include "TIADebug.hxx"
@@ -121,12 +122,12 @@ Debugger::Debugger(OSystem& osystem, Console& console)
     myBreakPoints(NULL),
     myReadTraps(NULL),
     myWriteTraps(NULL),
-    myWidth(1050),
-    myHeight(620),
+    myWidth(1080),
+    myHeight(720),
     myRewindManager(NULL)
 {
   // Init parser
-  myParser = new DebuggerParser(this);
+  myParser = new DebuggerParser(*this);
 
   // Create debugger subsystems
   myCpuDebug  = new CpuDebug(*this, myConsole);
@@ -165,13 +166,12 @@ Debugger::~Debugger()
 void Debugger::initialize()
 {
   // Get the dialog size
-  int w, h;
-  myOSystem->settings().getSize("debuggerres", w, h);
-  myWidth = BSPF_max(w, 0);
-  myHeight = BSPF_max(h, 0);
-  myWidth = BSPF_max(myWidth, 1050u);
-  myHeight = BSPF_max(myHeight, 700u);
-  myOSystem->settings().setSize("debuggerres", myWidth, myHeight);
+  const GUI::Size& size = myOSystem->settings().getSize("debuggerres");
+  myWidth = BSPF_max(size.w, 0);
+  myHeight = BSPF_max(size.h, 0);
+  myWidth = BSPF_max(myWidth, 1080u);
+  myHeight = BSPF_max(myHeight, 720u);
+  myOSystem->settings().setValue("debuggerres", GUI::Size(myWidth, myHeight));
 
   const GUI::Rect& r = getDialogBounds();
 
@@ -181,6 +181,7 @@ void Debugger::initialize()
   myBaseDialog = myDialog;
 
   myRewindManager = new RewindManager(*myOSystem, myDialog->rewindButton());
+  myCartDebug->setDebugWidget(&(myDialog->cartDebug()));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -204,7 +205,7 @@ bool Debugger::start(const string& message, int address)
     if(address > -1)
       buf << valueToString(address);
 
-    myDialog->message().setEditString(buf.str());
+    myDialog->message().setText(buf.str());
     return true;
   }
   return false;
@@ -243,14 +244,7 @@ string Debugger::autoExec()
       << myParser->exec(autoexec) << endl;
 
   // Also, "romname.stella" if present
-  string file = myOSystem->romFile();
-  string::size_type pos;
-  if( (pos = file.find_last_of('.')) != string::npos )
-    file.replace(pos, file.size(), ".stella");
-  else
-    file += ".stella";
-
-  FilesystemNode romname(file);
+  FilesystemNode romname(myOSystem->romFile().getPathWithExt(".stella"));
   buf << myParser->exec(romname) << endl;
 
   // Init builtins
@@ -313,6 +307,9 @@ string Debugger::valueToString(int value, BaseFormat outputBase) const
     case kBASE_16_4:  // base 16: 4 bytes wide
       BSPF_snprintf(vToS_buf, 5, "%04X", value);
       break;
+    case kBASE_16_8:  // base 16: 8 bytes wide
+      BSPF_snprintf(vToS_buf, 9, "%08X", value);
+      break;
 
     case kBASE_16:    // base 16: 2, 4, 8 bytes (depending on value)
     default:
@@ -809,19 +806,6 @@ void Debugger::getCompletions(const char* in, StringList& list) const
       list.push_back(pseudo_registers[i][0]);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::saveROM(const string& filename) const
-{
-  string path = AbstractFilesystemNode::getAbsolutePath(filename, "~", "a26");
-  FilesystemNode node(path);
-
-  ofstream out(node.getPath().c_str(), ios::out | ios::binary);
-  if(out.is_open() && myConsole.cartridge().save(out))
-    return node.getRelativePath();
-  else
-    return "";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::lockBankswitchState()
 {
