@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PackedBitArray.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: PackedBitArray.cxx 2725 2013-05-07 18:53:21Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -22,8 +22,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PackedBitArray::PackedBitArray(uInt32 length)
-  : size(length),
-    words(length / wordSize + 1)
+  : words(length / wordSize + 1)
 {
   bits = new uInt32[ words ];
 
