@@ -8,19 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RiotDebug.cxx 2488 2012-05-20 18:26:59Z stephena $
+// $Id: RiotDebug.cxx 2614 2013-02-17 22:33:53Z stephena $
 //============================================================================
 
 #include <sstream>
 
 #include "System.hxx"
-#include "M6532.hxx"
 #include "TIA.hxx"
 #include "Debugger.hxx"
 #include "Switches.hxx"
@@ -129,7 +128,7 @@ uInt8 RiotDebug::swacnt(int newVal)
   if(newVal > -1)
     mySystem.poke(0x281, newVal);
 
-  return mySystem.peek(0x281);
+  return mySystem.m6532().myDDRA;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -138,7 +137,7 @@ uInt8 RiotDebug::swbcnt(int newVal)
   if(newVal > -1)
     mySystem.poke(0x283, newVal);
 
-  return mySystem.peek(0x283);
+  return mySystem.m6532().myDDRB;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -196,24 +195,6 @@ uInt8 RiotDebug::tim1024T(int newVal)
   return mySystem.m6532().myOutTimer[3];
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 RiotDebug::intim()
-{
-  return mySystem.peek(0x284);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 RiotDebug::timint()
-{
-  return mySystem.peek(0x285);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Int32 RiotDebug::timClocks()
-{
-  return mySystem.m6532().timerClocks();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Controller& RiotDebug::controller(Controller::Jack jack) const
 {
