@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CpuWidget.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: CpuWidget.cxx 2743 2013-05-29 16:27:12Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -138,7 +138,7 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
   switch(cmd)
   {
-    case kDGItemDataChangedCmd:
+    case DataGridWidget::kItemDataChangedCmd:
       switch(id)
       {
         case kPCRegID:
@@ -190,7 +190,7 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       }
       break;
 
-    case kTWItemDataChangedCmd:
+    case ToggleWidget::kItemDataChangedCmd:
     {
       bool state = myPSRegister->getSelectedState();
 
@@ -309,5 +309,5 @@ void CpuWidget::fillGrid()
     changed.push_back(state.PSbits[i] != oldstate.PSbits[i]);
 
   myPSRegister->setState(state.PSbits, changed);
-  myPCLabel->setEditString(dbg.cartDebug().getLabel(state.PC, true));
+  myPCLabel->setText(dbg.cartDebug().getLabel(state.PC, true));
 }
