@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerDialog.cxx 2356 2012-01-14 22:00:54Z stephena $
+// $Id: DebuggerDialog.cxx 2743 2013-05-29 16:27:12Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -33,6 +33,7 @@
 #include "RiotWidget.hxx"
 #include "RomWidget.hxx"
 #include "TiaWidget.hxx"
+#include "CartDebugWidget.hxx"
 #include "DataGridOpsWidget.hxx"
 #include "EditTextWidget.hxx"
 #include "MessageBox.hxx"
@@ -47,6 +48,7 @@ DebuggerDialog::DebuggerDialog(OSystem* osystem, DialogContainer* parent,
                                int x, int y, int w, int h)
   : Dialog(osystem, parent, x, y, w, h, true),  // use base surface
     myTab(NULL),
+    myRomTab(NULL),
     myFatalError(NULL)
 {
   addTiaArea();
@@ -72,9 +74,9 @@ void DebuggerDialog::loadConfig()
   myTiaZoom->loadConfig();
   myCpu->loadConfig();
   myRam->loadConfig();
-  myRom->loadConfig();
+  myRomTab->loadConfig();
 
-  myMessageBox->setEditString("");
+  myMessageBox->setText("");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -145,6 +147,10 @@ void DebuggerDialog::handleCommand(CommandSender* sender, int cmd,
       doExitRom();
       break;
 
+    case RomWidget::kInvalidateListing:
+      myRom->invalidate();
+      break;
+
     default:
       Dialog::handleCommand(sender, cmd, data, id);
   }
@@ -157,9 +163,9 @@ void DebuggerDialog::showFatalMessage(const string& msg)
 
   delete myFatalError;
   myFatalError =
-    new MessageBox(this, instance().consoleFont(), msg,
-                   r.width(), r.height(), kDDExitFatalCmd,
-                   "Exit ROM", "Continue");
+    new GUI::MessageBox(this, instance().consoleFont(), msg,
+                        r.width(), r.height(), kDDExitFatalCmd,
+                        "Exit ROM", "Continue");
   myFatalError->show();
 }
 
@@ -176,12 +182,14 @@ void DebuggerDialog::addTiaArea()
 void DebuggerDialog::addTabArea()
 {
   const GUI::Rect& r = instance().debugger().getTabBounds();
-
   const int vBorder = 4;
 
   // The tab widget
+  // Since there are two tab widgets in this dialog, we specifically
+  // assign an ID of 0
   myTab = new TabWidget(this, instance().consoleFont(), r.left, r.top + vBorder,
                         r.width(), r.height() - vBorder);
+  myTab->setID(0);
   addTabWidget(myTab);
 
   const int widWidth  = r.width() - vBorder;
@@ -193,28 +201,28 @@ void DebuggerDialog::addTabArea()
   myPrompt = new PromptWidget(myTab, instance().consoleFont(),
                               2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, myPrompt);
-  addToFocusList(myPrompt->getFocusList(), tabID);
+  addToFocusList(myPrompt->getFocusList(), myTab, tabID);
 
   // The TIA tab
   tabID = myTab->addTab("TIA");
   TiaWidget* tia = new TiaWidget(myTab, instance().consoleFont(),
                                  2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, tia);
-  addToFocusList(tia->getFocusList(), tabID);
+  addToFocusList(tia->getFocusList(), myTab, tabID);
 
   // The input/output tab (includes RIOT and INPTx from TIA)
   tabID = myTab->addTab("I/O");
   RiotWidget* riot = new RiotWidget(myTab, instance().consoleFont(),
                                     2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, riot);
-  addToFocusList(riot->getFocusList(), tabID);
+  addToFocusList(riot->getFocusList(), myTab, tabID);
 
   // The Audio tab
   tabID = myTab->addTab("Audio");
   AudioWidget* aud = new AudioWidget(myTab, instance().consoleFont(),
                                      2, 2, widWidth, widHeight);
   myTab->setParentWidget(tabID, aud);
-  addToFocusList(aud->getFocusList(), tabID);
+  addToFocusList(aud->getFocusList(), myTab, tabID);
 
   myTab->setActiveTab(0);
 }
@@ -248,6 +256,8 @@ void DebuggerDialog::addStatusArea()
 void DebuggerDialog::addRomArea()
 {
   const GUI::Rect& r = instance().debugger().getRomBounds();
+  const int vBorder = 4;
+
   int xpos, ypos;
 
   xpos = r.left + 10;  ypos = 10;
@@ -289,14 +299,46 @@ void DebuggerDialog::addRomArea()
                      bwidth, bheight, "<", kDDRewindCmd);
   myRewindButton->clearFlags(WIDGET_ENABLED);
 
-  xpos = r.left + 10;  ypos += myRam->getHeight() + 5;
-  myRom = new RomWidget(this, instance().consoleFont(), xpos, ypos);
-  addToFocusList(myRom->getFocusList());
-
   // Add the DataGridOpsWidget to any widgets which contain a
   // DataGridWidget which we want controlled
   myCpu->setOpsWidget(ops);
   myRam->setOpsWidget(ops);
+
+  ////////////////////////////////////////////////////////////////////
+  // Disassembly area
+
+  xpos = r.left + vBorder;  ypos += myRam->getHeight() + 5;
+  const int tabWidth  = r.width() - vBorder - 1;
+  const int tabHeight = r.height() - ypos - 1;
+  int tabID;
+
+  // Since there are two tab widgets in this dialog, we specifically
+  // assign an ID of 1
+  myRomTab = new TabWidget(
+      this, instance().consoleFont(), xpos, ypos, tabWidth, tabHeight);
+  myRomTab->setID(1);
+  addTabWidget(myRomTab);
+
+  // The main disassembly tab
+  tabID = myRomTab->addTab("   Disassembly   ");
+  myRom = new RomWidget(myRomTab, instance().consoleFont(),
+                        2, 2, tabWidth - 1,
+                        tabHeight - myRomTab->getTabHeight() - 2);
+  myRomTab->setParentWidget(tabID, myRom);
+  addToFocusList(myRom->getFocusList(), myRomTab, tabID);
+
+  // The 'cart-specific' information tab
+  tabID = myRomTab->addTab(instance().console().cartridge().name());
+  myCartDebug = instance().console().cartridge().debugWidget(
+        myRomTab, instance().consoleFont(), 2, 2, tabWidth - 1,
+        tabHeight - myRomTab->getTabHeight() - 2);
+  if(myCartDebug)  // TODO - make this always non-null
+  {
+    myRomTab->setParentWidget(tabID, myCartDebug);
+    addToFocusList(myCartDebug->getFocusList(), myRomTab, tabID);
+  }
+
+  myRomTab->setActiveTab(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
