@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RamWidget.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: RamWidget.cxx 2743 2013-05-29 16:27:12Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -46,7 +46,7 @@ RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight(),
-            bwidth  = 44,//font.getStringWidth("Undo  "),
+            bwidth  = font.getStringWidth("Compare "),
             bheight = lineHeight + 2;
   int xpos, ypos, lwidth;
 
@@ -66,22 +66,22 @@ RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 
   ypos += bheight + 4;
   myRevertButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                    "Rev", kRevertCmd);
+                                    "Revert", kRevertCmd);
   myRevertButton->setTarget(this);
 
   ypos += 2 * bheight + 2;
   mySearchButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                    "Srch", kSearchCmd);
+                                    "Search", kSearchCmd);
   mySearchButton->setTarget(this);
 
   ypos += bheight + 4;
   myCompareButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                     "Cmp", kCmpCmd);
+                                     "Compare", kCmpCmd);
   myCompareButton->setTarget(this);
 
   ypos += bheight + 4;
   myRestartButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                     "Rset", kRestartCmd);
+                                     "Reset", kRestartCmd);
   myRestartButton->setTarget(this);
 
   // Labels for RAM grid
@@ -164,7 +164,7 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   const CartState& state = (CartState&) dbg.getState();
   switch(cmd)
   {
-    case kDGItemDataChangedCmd:
+    case DataGridWidget::kItemDataChangedCmd:
     {
       addr  = myRamGrid->getSelectedAddr();
       value = myRamGrid->getSelectedValue();
@@ -182,21 +182,21 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       myUndoAddress = addr;
       myUndoValue = oldval;
 
-      myDecValue->setEditString(instance().debugger().valueToString(value, kBASE_10));
-      myBinValue->setEditString(instance().debugger().valueToString(value, kBASE_2));
+      myDecValue->setText(instance().debugger().valueToString(value, kBASE_10));
+      myBinValue->setText(instance().debugger().valueToString(value, kBASE_2));
       myRevertButton->setEnabled(true);
       myUndoButton->setEnabled(true);
       break;
     }
 
-    case kDGSelectionChangedCmd:
+    case DataGridWidget::kSelectionChangedCmd:
     {
       addr  = myRamGrid->getSelectedAddr();
       value = myRamGrid->getSelectedValue();
 
-      myLabel->setEditString(dbg.getLabel(state.rport[addr], true));
-      myDecValue->setEditString(instance().debugger().valueToString(value, kBASE_10));
-      myBinValue->setEditString(instance().debugger().valueToString(value, kBASE_2));
+      myLabel->setText(dbg.getLabel(state.rport[addr], true));
+      myDecValue->setText(instance().debugger().valueToString(value, kBASE_10));
+      myBinValue->setText(instance().debugger().valueToString(value, kBASE_2));
       break;
     }
 
@@ -230,7 +230,7 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       if(result != "")
         myInputBox->setTitle(result);
       else
-        parent().removeDialog();
+        dialog().close();
       break;
     }
 
@@ -240,7 +240,7 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       if(result != "")
         myInputBox->setTitle(result);
       else
-        parent().removeDialog();
+        dialog().close();
       break;
     }
 
@@ -320,7 +320,7 @@ void RamWidget::showInputBox(int cmd)
   uInt32 x = getAbsX() + ((getWidth() - myInputBox->getWidth()) >> 1);
   uInt32 y = getAbsY() + ((getHeight() - myInputBox->getHeight()) >> 1);
   myInputBox->show(x, y);
-  myInputBox->setEditString("");
+  myInputBox->setText("");
   myInputBox->setTitle("");
   myInputBox->setFocus(0);
   myInputBox->setEmitSignal(cmd);
