@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RiotWidget.cxx 2346 2012-01-08 19:58:55Z stephena $
+// $Id: RiotWidget.cxx 2732 2013-05-09 14:22:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -64,7 +64,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& font,
             lineHeight = font.getLineHeight();
   int xpos = 10, ypos = 25, lwidth = 9 * fontWidth, col = 0;
   StaticTextWidget* t;
-  StringMap items;
+  VariantList items;
 
   // Set the strings to be used in the various bit registers
   // We only do this once because it's the state that changes, not the strings
@@ -320,7 +320,7 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
   switch(cmd)
   {
-    case kDGItemDataChangedCmd:
+    case DataGridWidget::kItemDataChangedCmd:
       switch(id)
       {
         case kTimWriteID:
@@ -343,7 +343,7 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       }
       break;
 
-    case kTWItemDataChangedCmd:
+    case ToggleWidget::kItemDataChangedCmd:
       switch(id)
       {
         case kSWCHABitsID:
@@ -378,11 +378,11 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kP0DiffChanged:
-      riot.diffP0(myP0Diff->getSelectedTag() != "b");
+      riot.diffP0(myP0Diff->getSelectedTag().toString() != "b");
       break;
 
     case kP1DiffChanged:
-      riot.diffP1(myP1Diff->getSelectedTag() != "b");
+      riot.diffP1(myP1Diff->getSelectedTag().toString() != "b");
       break;
 
     case kTVTypeChanged:
