@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaInfoWidget.cxx 2343 2012-01-08 16:55:10Z stephena $
+// $Id: TiaInfoWidget.cxx 2743 2013-05-29 16:27:12Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -125,15 +125,15 @@ void TiaInfoWidget::loadConfig()
   Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
 
-  myFrameCount->setEditString(dbg.valueToString(tia.frameCount(), kBASE_10));
-  myFrameCycles->setEditString(dbg.valueToString(dbg.cycles(), kBASE_10));
+  myFrameCount->setText(dbg.valueToString(tia.frameCount(), kBASE_10));
+  myFrameCycles->setText(dbg.valueToString(dbg.cycles(), kBASE_10));
 
   myVSync->setState(tia.vsync());
   myVBlank->setState(tia.vblank());
 
   int clk = tia.clocksThisLine();
-  myScanlineCount->setEditString(dbg.valueToString(tia.scanlines(), kBASE_10));
-  myScanlineCycles->setEditString(dbg.valueToString(clk/3, kBASE_10));
-  myPixelPosition->setEditString(dbg.valueToString(clk-68, kBASE_10));
-  myColorClocks->setEditString(dbg.valueToString(clk, kBASE_10));
+  myScanlineCount->setText(dbg.valueToString(tia.scanlines(), kBASE_10));
+  myScanlineCycles->setText(dbg.valueToString(clk/3, kBASE_10));
+  myPixelPosition->setText(dbg.valueToString(clk-68, kBASE_10));
+  myColorClocks->setText(dbg.valueToString(clk, kBASE_10));
 }
