@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaOutputWidget.cxx 2377 2012-01-29 22:54:23Z stephena $
+// $Id: TiaOutputWidget.cxx 2732 2013-05-09 14:22:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -46,7 +46,7 @@ TiaOutputWidget::TiaOutputWidget(GuiObject* boss, const GUI::Font& font,
   _type = kTiaOutputWidget;
 
   // Create context menu for commands
-  StringMap l;
+  VariantList l;
   l.push_back("Fill to scanline", "scanline");
   l.push_back("Set breakpoint", "bp");
   l.push_back("Set zoom position", "zoom");
@@ -87,9 +87,9 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
 
   switch(cmd)
   {
-    case kCMenuItemSelectedCmd:
+    case ContextMenu::kItemSelectedCmd:
     {
-      const string& rmb = myMenu->getSelectedTag();
+      const string& rmb = myMenu->getSelectedTag().toString();
 
       if(rmb == "scanline")
       {
