@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx 2343 2012-01-08 16:55:10Z stephena $
+// $Id: TiaWidget.cxx 2743 2013-05-29 16:27:12Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -585,7 +585,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       tia.strobeCxclr();
       break;
 
-    case kDGItemDataChangedCmd:
+    case DataGridWidget::kItemDataChangedCmd:
       switch(id)
       {
         case kColorRegsID:
@@ -634,12 +634,12 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
         case kNusizP0ID:
           tia.nusizP0(myNusizP0->getSelectedValue());
-          myNusizP0Text->setEditString(tia.nusizP0String());
+          myNusizP0Text->setText(tia.nusizP0String());
           break;
 
         case kNusizP1ID:
           tia.nusizP1(myNusizP1->getSelectedValue());
-          myNusizP1Text->setEditString(tia.nusizP1String());
+          myNusizP1Text->setText(tia.nusizP1String());
           break;
 
         case kNusizM0ID:
@@ -660,7 +660,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       }
       break;
 
-    case kTWItemDataChangedCmd:
+    case ToggleWidget::kItemDataChangedCmd:
       switch(id)
       {
         case kGRP0ID:
@@ -863,8 +863,9 @@ void TiaWidget::fillGrid()
   myDelP0->setState(tia.vdelP0());
 
   // NUSIZ0 (player portion)
-  myNusizP0->setList(0, state.size[P0], state.size[P0] != oldstate.size[P0]);
-  myNusizP0Text->setEditString(tia.nusizP0String());
+  bool nusiz0changed = state.size[P0] != oldstate.size[P0];
+  myNusizP0->setList(0, state.size[P0], nusiz0changed);
+  myNusizP0Text->setText(tia.nusizP0String(), nusiz0changed);
 
   ////////////////////////////
   // P1 register info
@@ -884,8 +885,9 @@ void TiaWidget::fillGrid()
   myDelP1->setState(tia.vdelP1());
 
   // NUSIZ1 (player portion)
-  myNusizP1->setList(0, state.size[P1], state.size[P1] != oldstate.size[P1]);
-  myNusizP1Text->setEditString(tia.nusizP1String());
+  bool nusiz1changed = state.size[P1] != oldstate.size[P1];
+  myNusizP1->setList(0, state.size[P1], nusiz1changed);
+  myNusizP1Text->setText(tia.nusizP1String(), nusiz1changed);
 
   ////////////////////////////
   // M0 register info
