@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ToggleBitWidget.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: ToggleBitWidget.cxx 2704 2013-04-22 16:41:05Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -38,6 +38,16 @@ ToggleBitWidget::ToggleBitWidget(GuiObject* boss, const GUI::Font& font,
   _rowHeight = font.getLineHeight();
   _colWidth  = colchars * font.getMaxCharWidth() + 8;
 
+  // Make sure all lists contain some default values
+  int size = _rows * _cols;
+  while(size--)
+  {
+    _offList.push_back("0");
+    _onList.push_back("1");
+    _stateList.push_back(false);
+    _changedList.push_back(false);
+  }
+
   // Calculate real dimensions
   _w = _colWidth  * cols + 1;
   _h = _rowHeight * rows + 1;
