@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDPCPlus.cxx 2526 2012-06-04 19:20:27Z stephena $
+// $Id: CartDPCPlus.cxx 2703 2013-04-21 20:02:55Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -50,18 +50,13 @@ CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size,
   // Pointer to the display RAM
   myDisplayImage = myDPCRAM + 0xC00;
 
-  // Pointer to the Frequency ROM (1K @ 28K offset)
-  myFrequencyImage = myProgramImage + 0x7000;
+  // Pointer to the Frequency RAM
+  myFrequencyImage = myDisplayImage + 0x1000;
 
   // If the image is larger than 29K, we assume any excess at the
   // beginning is ARM code, and skip over it
   if(size > 29 * 1024)
-  {
-    int offset = size - 29 * 1024;
-    myProgramImage   += offset;
-//    myDisplayImage   += offset;
-//    myFrequencyImage += offset;
-  }
+    myProgramImage += (size - 29 * 1024);
 
 #ifdef THUMB_SUPPORT
   // Create Thumbulator ARM emulator
@@ -108,7 +103,7 @@ void CartridgeDPCPlus::setInitialState()
   memcpy(myDisplayImage, myProgramImage + 0x6000, 0x1400);
 
   // Initialize the DPC data fetcher registers
-  for(uInt16 i = 0; i < 8; ++i)
+  for(int i = 0; i < 8; ++i)
     myTops[i] = myBottoms[i] = myCounters[i] = myFractionalIncrements[i] = 
     myFractionalCounters[i] = 0;
 
