@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CompuMate.cxx 2447 2012-04-19 20:11:16Z stephena $
+// $Id: CompuMate.cxx 2705 2013-04-23 15:57:33Z stephena $
 //============================================================================
 
 #include "Control.hxx"
@@ -39,8 +39,20 @@ CompuMate::CompuMate(CartridgeCM& cart, const Event& event,
   myLeftController->myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
   myRightController->myAnalogPinValue[Controller::Nine] = Controller::minimumResistance;
   myRightController->myAnalogPinValue[Controller::Five] = Controller::maximumResistance;
+}
 
-  myKeyTable = event.getKeys();
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CompuMate::enableKeyHandling(bool enable)
+{
+  if(enable)
+    myKeyTable = myEvent.getKeys();
+  else
+  {
+    for(uInt32 i = 0; i < KBDK_LAST; ++i)
+      myInternalKeyTable[i] = false;
+
+    myKeyTable = myInternalKeyTable;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
