@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Console.cxx 2527 2012-06-05 16:32:35Z stephena $
+// $Id: Console.cxx 2726 2013-05-08 23:34:42Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -75,9 +75,9 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
     mySystem(0),
     myCart(cart),
     myCMHandler(0),
-    myDisplayFormat("NTSC"),
-    myFramerate(60.0),
-    myCurrentFormat(0),
+    myDisplayFormat(""),  // Unknown TV format @ start
+    myFramerate(0.0),     // Unknown framerate @ start
+    myCurrentFormat(0),   // Unknown format @ start
     myUserPaletteDefined(false)
 {
   // Load user-defined palette for this ROM
@@ -112,27 +112,26 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
   // Auto-detect NTSC/PAL mode if it's requested
   string autodetected = "";
   myDisplayFormat = myProperties.get(Display_Format);
-  if(myDisplayFormat == "AUTO-DETECT" ||
-     myOSystem->settings().getBool("rominfo"))
+  if(myDisplayFormat == "AUTO" || myOSystem->settings().getBool("rominfo"))
   {
     // Run the TIA, looking for PAL scanline patterns
     // We turn off the SuperCharger progress bars, otherwise the SC BIOS
     // will take over 250 frames!
     // The 'fastscbios' option must be changed before the system is reset
     bool fastscbios = myOSystem->settings().getBool("fastscbios");
-    myOSystem->settings().setBool("fastscbios", true);
+    myOSystem->settings().setValue("fastscbios", true);
     mySystem->reset(true);  // autodetect in reset enabled
     for(int i = 0; i < 60; ++i)
       myTIA->update();
     myDisplayFormat = myTIA->isPAL() ? "PAL" : "NTSC";
-    if(myProperties.get(Display_Format) == "AUTO-DETECT")
+    if(myProperties.get(Display_Format) == "AUTO")
     {
       autodetected = "*";
       myCurrentFormat = 0;
     }
 
     // Don't forget to reset the SC progress bars again
-    myOSystem->settings().setBool("fastscbios", fastscbios);
+    myOSystem->settings().setValue("fastscbios", fastscbios);
   }
   myConsoleInfo.DisplayFormat = myDisplayFormat + autodetected;
 
@@ -164,11 +163,6 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
   // Reset the system to its power-on state
   mySystem->reset();
 
-#ifdef DEBUGGER_SUPPORT
-  myOSystem->createDebugger(*this);
-  m6502->attach(myOSystem->debugger());
-#endif
-
   // Finally, add remaining info about the console
   myConsoleInfo.CartName   = myProperties.get(Cartridge_Name);
   myConsoleInfo.CartMD5    = myProperties.get(Cartridge_MD5);
@@ -251,7 +245,7 @@ void Console::toggleFormat(int direction)
       myTIA->update();
       myDisplayFormat = myTIA->isPAL() ? "PAL" : "NTSC";
       message = "Auto-detect mode: " + myDisplayFormat;
-      saveformat = "AUTO-DETECT";
+      saveformat = "AUTO";
       break;
     case 1:
       saveformat = myDisplayFormat  = "NTSC";
@@ -292,7 +286,7 @@ void Console::toggleFormat(int direction)
 void Console::toggleColorLoss()
 {
   bool colorloss = !myOSystem->settings().getBool("colorloss");
-  myOSystem->settings().setBool("colorloss", colorloss);
+  myOSystem->settings().setValue("colorloss", colorloss);
   myTIA->enableColorLoss(colorloss);
 
   string message = string("PAL color-loss ") +
@@ -343,7 +337,7 @@ void Console::togglePalette()
     message = "Standard Stella palette";
   }
 
-  myOSystem->settings().setString("palette", palette);
+  myOSystem->settings().setValue("palette", palette);
   myOSystem->frameBuffer().showMessage(message);
 
   setPalette(palette);
@@ -597,19 +591,14 @@ void Console::setTIAProperties()
 
   // Make sure these values fit within the bounds of the desktop
   // If not, attempt to center vertically
-  if(height <= myOSystem->desktopHeight())
-  {
-    myTIA->setYStart(ystart);
-    myTIA->setHeight(height);
-  }
-  else
+  if(height > myOSystem->desktopHeight())
   {
     ystart += height - myOSystem->desktopHeight();
     ystart = BSPF_min(ystart, 64u);
     height = myOSystem->desktopHeight();
-    myTIA->setYStart(ystart);
-    myTIA->setHeight(height);
   }
+  myTIA->setYStart(ystart);
+  myTIA->setHeight(height);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -744,16 +733,16 @@ void Console::setControllers(const string& rommd5)
   }
   else if(right == "ATARIVOX")
   {
-    const string& eepromfile = myOSystem->eepromDir() + "atarivox_eeprom.dat";
+    const string& nvramfile = myOSystem->nvramDir() + "atarivox_eeprom.dat";
     myControllers[rightPort] = new AtariVox(Controller::Right, myEvent,
                    *mySystem, myOSystem->serialPort(),
-                   myOSystem->settings().getString("avoxport"), eepromfile);
+                   myOSystem->settings().getString("avoxport"), nvramfile);
   }
   else if(right == "SAVEKEY")
   {
-    const string& eepromfile = myOSystem->eepromDir() + "savekey_eeprom.dat";
+    const string& nvramfile = myOSystem->nvramDir() + "savekey_eeprom.dat";
     myControllers[rightPort] = new SaveKey(Controller::Right, myEvent, *mySystem,
-                                           eepromfile);
+                                           nvramfile);
   }
   else if(right == "GENESIS")
   {
@@ -926,6 +915,23 @@ void Console::toggleFixedColors() const
     myOSystem->frameBuffer().showMessage("Fixed debug colors disabled");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::addDebugger()
+{
+#ifdef DEBUGGER_SUPPORT
+  myOSystem->createDebugger(*this);
+  mySystem->m6502().attach(myOSystem->debugger());
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::stateChanged(EventHandler::State state)
+{
+  // For now, only the CompuMate cares about state changes
+  if(myCMHandler)
+    myCMHandler->enableKeyHandling(state == EventHandler::S_EMULATE);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 Console::ourNTSCPalette[256] = {
   0x000000, 0, 0x4a4a4a, 0, 0x6f6f6f, 0, 0x8e8e8e, 0,
