@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventHandler.cxx 2527 2012-06-05 16:32:35Z stephena $
+// $Id: EventHandler.cxx 2753 2013-06-21 12:15:32Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -39,10 +39,9 @@
 #include "ListWidget.hxx"
 #include "ScrollBarWidget.hxx"
 #include "Settings.hxx"
-#include "Snapshot.hxx"
 #include "Sound.hxx"
 #include "StateManager.hxx"
-#include "Switches.hxx"
+#include "M6532.hxx"
 #include "MouseControl.hxx"
 
 #include "EventHandler.hxx"
@@ -136,7 +135,7 @@ void EventHandler::setupJoysticks()
   // Initialize the joystick subsystem
   if((SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1) || (SDL_NumJoysticks() <= 0))
   {
-    myOSystem->logMessage("No joysticks present.\n\n", 1);
+    myOSystem->logMessage("No joysticks present.", 1);
     return;
   }
 
@@ -220,7 +219,7 @@ void EventHandler::setupJoysticks()
     buf << "  " << i << ": " << myJoysticks[i].about() << endl << endl;
   myOSystem->logMessage(buf.str(), 1);
 #else
-  myOSystem->logMessage("No joysticks present.\n\n", 1);
+  myOSystem->logMessage("No joysticks present.", 1);
 #endif
 }
 
@@ -275,7 +274,7 @@ void EventHandler::mapStelladaptors(const string& saport)
       }
     }
   }
-  myOSystem->settings().setString("saport", saport);
+  myOSystem->settings().setValue("saport", saport);
 
   // We're potentially swapping out an input device behind the back of
   // the Event system, so we make sure all Stelladaptor-generated events
@@ -594,8 +593,7 @@ void EventHandler::poll(uInt64 time)
                 break;
 
               case KBDK_r:  // Ctrl-r reloads the currently loaded ROM
-                myOSystem->deleteConsole();
-                myOSystem->createConsole();
+                myOSystem->reloadConsole();
                 break;
 
               case KBDK_PAGEUP:    // Ctrl-PageUp increases Height
@@ -955,9 +953,7 @@ void EventHandler::poll(uInt64 time)
   // related to emulation
   if(myState == S_EMULATE)
   {
-    myOSystem->console().controller(Controller::Left).update();
-    myOSystem->console().controller(Controller::Right).update();
-    myOSystem->console().switches().update();
+    myOSystem->console().riot().update();
 
 #if 0
     // Now check if the StateManager should be saving or loading state
@@ -1725,7 +1721,7 @@ void EventHandler::saveKeyMapping()
     for(int i = 0; i < KBDK_LAST; ++i)
       keybuf << ":" << myKeyTable[i][mode];
 
-  myOSystem->settings().setString("keymap", keybuf.str());
+  myOSystem->settings().setValue("keymap", keybuf.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1760,7 +1756,7 @@ void EventHandler::saveJoyMapping()
   for(iter = myJoystickMap.begin(); iter != myJoystickMap.end(); ++iter)
     joybuf << "^" << iter->second;
 
-  myOSystem->settings().setString("joymap", joybuf.str());
+  myOSystem->settings().setValue("joymap", joybuf.str());
 #endif
 }
 
@@ -1778,7 +1774,7 @@ void EventHandler::saveComboMapping()
     for(int j = 1; j < kEventsPerCombo; ++j)
       buf << "," << myComboTable[i][j];
   }
-  myOSystem->settings().setString("combomap", buf.str());
+  myOSystem->settings().setValue("combomap", buf.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1815,7 +1811,7 @@ void EventHandler::getActionList(EventMode mode, StringList& l) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::getComboList(EventMode, StringMap& l) const
+void EventHandler::getComboList(EventMode, VariantList& l) const
 {
   // For now, this only works in emulation mode
 
@@ -1952,8 +1948,10 @@ void EventHandler::takeSnapshot(uInt32 number)
   // Figure out the correct snapshot name
   string filename;
   bool showmessage = number == 0;
-  string sspath = myOSystem->snapshotDir() +
-      myOSystem->console().properties().get(Cartridge_Name);
+  string sspath = myOSystem->snapshotSaveDir() +
+      (myOSystem->settings().getString("snapname") != "int" ?
+          myOSystem->romFile().getNameWithExt("")
+        : myOSystem->console().properties().get(Cartridge_Name));
 
   // Check whether we want multiple snapshots created
   if(number > 0)
@@ -1988,9 +1986,10 @@ void EventHandler::takeSnapshot(uInt32 number)
   // Now create a PNG snapshot
   if(myOSystem->settings().getBool("ss1x"))
   {
-    string msg = Snapshot::savePNG(myOSystem->frameBuffer(),
-                   myOSystem->console().tia(),
-                   myOSystem->console().properties(), filename);
+    string msg =
+      myOSystem->png().saveImage(filename, myOSystem->frameBuffer(),
+                                 myOSystem->console().tia(),
+                                 myOSystem->console().properties());
     if(showmessage)
       myOSystem->frameBuffer().showMessage(msg);
   }
@@ -1999,8 +1998,9 @@ void EventHandler::takeSnapshot(uInt32 number)
     // Make sure we have a 'clean' image, with no onscreen messages
     myOSystem->frameBuffer().enableMessages(false);
 
-    string msg = Snapshot::savePNG(myOSystem->frameBuffer(),
-                   myOSystem->console().properties(), filename);
+    string msg =
+      myOSystem->png().saveImage(filename, myOSystem->frameBuffer(),
+                                 myOSystem->console().properties());
 
     // Re-enable old messages
     myOSystem->frameBuffer().enableMessages(true);
@@ -2159,6 +2159,10 @@ void EventHandler::setEventState(State state)
     myOSystem->frameBuffer().stateChanged(myState);
     myOSystem->frameBuffer().setCursorState();
   }
+  if(&myOSystem->console())
+  {
+    myOSystem->console().stateChanged(myState);
+  }
 
   // Always clear any pending events when changing states
   myEvent.clear();
