@@ -8,20 +8,23 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNode.cxx 2477 2012-05-16 20:52:33Z stephena $
+// $Id: FSNode.cxx 2753 2013-06-21 12:15:32Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
+#include <zlib.h>
+
 #include "bspf.hxx"
 #include "SharedPtr.hxx"
+#include "FSNodeFactory.hxx"
 #include "FSNode.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -30,7 +33,7 @@ FilesystemNode::FilesystemNode()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FilesystemNode::FilesystemNode(AbstractFilesystemNode *realNode) 
+FilesystemNode::FilesystemNode(AbstractFSNode *realNode) 
   : _realNode(realNode)
 {
 }
@@ -38,31 +41,21 @@ FilesystemNode::FilesystemNode(AbstractFilesystemNode *realNode)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FilesystemNode::FilesystemNode(const string& p)
 {
-  AbstractFilesystemNode* tmp = 0;
-  if (p.empty() || p == "." || p == "~")
-    tmp = AbstractFilesystemNode::makeHomeDirectoryFileNode();
+  AbstractFSNode* tmp = 0;
+
+  // Is this potentially a ZIP archive?
+  if(BSPF_containsIgnoreCase(p, ".zip"))
+    tmp = FilesystemNodeFactory::create(p, FilesystemNodeFactory::ZIP);
   else
-    tmp = AbstractFilesystemNode::makeFileNodePath(p);
+    tmp = FilesystemNodeFactory::create(p, FilesystemNodeFactory::SYSTEM);
 
-  _realNode = Common::SharedPtr<AbstractFilesystemNode>(tmp);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FilesystemNode::operator<(const FilesystemNode& node) const
-{
-  if (isDirectory() != node.isDirectory())
-    return isDirectory();
-
-  return BSPF_strcasecmp(getDisplayName().c_str(), node.getDisplayName().c_str()) < 0;
+  _realNode = Common::SharedPtr<AbstractFSNode>(tmp);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::exists() const
 {
-  if (_realNode == 0)
-    return false;
-
-  return _realNode->exists();
+  return _realNode ? _realNode->exists() : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -78,34 +71,62 @@ bool FilesystemNode::getChildren(FSList& fslist, ListMode mode, bool hidden) con
     return false;
 
   for (AbstractFSList::iterator i = tmp.begin(); i != tmp.end(); ++i)
-  {
     fslist.push_back(FilesystemNode(*i));
-  }
 
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& FilesystemNode::getDisplayName() const
+const string& FilesystemNode::getName() const
 {
-  assert(_realNode);
-  return _realNode->getDisplayName();
+  return _realNode->getName();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& FilesystemNode::getName() const
+const string& FilesystemNode::getPath() const
 {
-  assert(_realNode);
-  return _realNode->getName();
+  return _realNode->getPath();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string FilesystemNode::getShortPath() const
+{
+  return _realNode->getShortPath();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string FilesystemNode::getNameWithExt(const string& ext) const
+{
+  size_t pos = _realNode->getName().find_last_of("/\\");
+  string s = pos == string::npos ? _realNode->getName() :
+        _realNode->getName().substr(pos+1);
+
+  pos = s.find_last_of(".");
+  return (pos != string::npos) ? s.replace(pos, string::npos, ext) : s + ext;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string FilesystemNode::getPathWithExt(const string& ext) const
+{
+  string s = _realNode->getPath();
+
+  size_t pos = s.find_last_of(".");
+  return (pos != string::npos) ? s.replace(pos, string::npos, ext) : s + ext;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string FilesystemNode::getShortPathWithExt(const string& ext) const
+{
+  string s = _realNode->getShortPath();
+
+  size_t pos = s.find_last_of(".");
+  return (pos != string::npos) ? s.replace(pos, string::npos, ext) : s + ext;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::hasParent() const
 {
-  if (_realNode == 0)
-    return false;
-  
-  return _realNode->getParent() != 0;
+  return _realNode ? (_realNode->getParent() != 0) : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -114,59 +135,74 @@ FilesystemNode FilesystemNode::getParent() const
   if (_realNode == 0)
     return *this;
 
-  AbstractFilesystemNode* node = _realNode->getParent();
-  if (node == 0)
-    return *this;
-  else
-    return FilesystemNode(node);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& FilesystemNode::getPath() const
-{
-  assert(_realNode);
-  return _realNode->getPath();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string FilesystemNode::getRelativePath() const
-{
-  assert(_realNode);
-  return _realNode->getRelativePath();
+  AbstractFSNode* node = _realNode->getParent();
+  return (node == 0) ? *this : FilesystemNode(node);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::isDirectory() const
 {
-  if (_realNode == 0)
-    return false;
-
-  return _realNode->isDirectory();
+  return _realNode ? _realNode->isDirectory() : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::isFile() const
 {
-  if (_realNode == 0)
-    return false;
-
-  return _realNode->isFile();
+  return _realNode ? _realNode->isFile() : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::isReadable() const
 {
-  if (_realNode == 0)
-    return false;
-
-  return _realNode->isReadable();
+  return _realNode ? _realNode->isReadable() : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNode::isWritable() const
 {
-  if (_realNode == 0)
-    return false;
+  return _realNode ? _realNode->isWritable() : false;
+}
 
-  return _realNode->isWritable();
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool FilesystemNode::makeDir()
+{
+  return (_realNode && !_realNode->exists()) ? _realNode->makeDir() : false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool FilesystemNode::rename(const string& newfile)
+{
+  return (_realNode && _realNode->exists()) ? _realNode->rename(newfile) : false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt32 FilesystemNode::read(uInt8*& image) const
+{
+  uInt32 size = 0;
+
+  // First let the private subclass attempt to open the file
+  if((size = _realNode->read(image)) > 0)
+    return size;
+
+  // File must actually exist
+  if(!(exists() && isReadable()))
+    throw "File not found/readable";
+
+  // Otherwise, assume the file is either gzip'ed or not compressed at all
+  gzFile f = gzopen(getPath().c_str(), "rb");
+  if(f)
+  {
+    image = new uInt8[512 * 1024];
+    size = gzread(f, image, 512 * 1024);
+    gzclose(f);
+
+    if(size == 0)
+    {
+      delete[] image;  image = 0;
+      throw "Zero-byte file";
+    }
+    return size;
+  }
+  else
+    throw "ZLIB open/read error";
 }
