@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: MT24LC256.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: MT24LC256.cxx 2579 2013-01-04 19:49:01Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -33,9 +33,9 @@
   #define JPEE_LOG1(msg,arg1) sprintf(jpee_msg,(msg),(arg1)), jpee_logproc(jpee_msg)
   #define JPEE_LOG2(msg,arg1,arg2) sprintf(jpee_msg,(msg),(arg1),(arg2)), jpee_logproc(jpee_msg)
 #else
-  #define JPEE_LOG0(msg)
-  #define JPEE_LOG1(msg,arg1)
-  #define JPEE_LOG2(msg,arg1,arg2)
+  #define JPEE_LOG0(msg) { }
+  #define JPEE_LOG1(msg,arg1) { }
+  #define JPEE_LOG2(msg,arg1,arg2) { }
 #endif
 
 /*
