@@ -8,19 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystem.cxx 2477 2012-05-16 20:52:33Z stephena $
+// $Id: OSystem.cxx 2734 2013-05-16 19:11:44Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <sstream>
 #include <fstream>
-#include <zlib.h>
 
 #include <ctime>
 #ifdef HAVE_GETTIMEOFDAY
@@ -50,7 +49,6 @@
 #endif
 
 #include "FSNode.hxx"
-#include "unzip.h"
 #include "MD5.hxx"
 #include "Cart.hxx"
 #include "Settings.hxx"
@@ -72,8 +70,6 @@
 
 #include "OSystem.hxx"
 
-#define MAX_ROM_SIZE  512 * 1024
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 OSystem::OSystem()
   : myEventHandler(NULL),
@@ -90,6 +86,7 @@ OSystem::OSystem()
     myDebugger(NULL),
     myCheatManager(NULL),
     myStateManager(NULL),
+    myPNGLib(NULL),
     myQuitLoop(false),
     myRomFile(""),
     myRomMD5(""),
@@ -196,6 +193,8 @@ OSystem::~OSystem()
   delete myEventHandler;
 
   delete mySerialPort;
+  delete myPNGLib;
+  delete myZipHandler;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -208,12 +207,11 @@ bool OSystem::create()
       << "  Features: " << myFeatures << endl
       << "  " << myBuildInfo << endl << endl
       << "Base directory:       '"
-      << FilesystemNode(myBaseDir).getRelativePath() << "'" << endl
+      << FilesystemNode(myBaseDir).getShortPath() << "'" << endl
       << "Configuration file:   '"
-      << FilesystemNode(myConfigFile).getRelativePath() << "'" << endl
+      << FilesystemNode(myConfigFile).getShortPath() << "'" << endl
       << "User game properties: '"
-      << FilesystemNode(myPropertiesFile).getRelativePath() << "'" << endl
-      << endl;
+      << FilesystemNode(myPropertiesFile).getShortPath() << "'" << endl;
   logMessage(buf.str(), 1);
 
   // Get relevant information about the video hardware
@@ -305,6 +303,12 @@ bool OSystem::create()
   // Let the random class know about us; it needs access to getTicks()
   Random::setSystem(this);
 
+  // Create PNG handler
+  myPNGLib = new PNGLibrary();
+
+  // Create ZIP handler
+  myZipHandler = new ZipHandler();
+
   return true;
 }
 
@@ -343,28 +347,29 @@ void OSystem::setConfigPaths()
   FilesystemNode node;
   string s;
 
-  validatePath(myStateDir, "statedir", myBaseDir + "statedir");
-  validatePath(mySnapshotDir, "snapdir", defaultSnapDir());
-  validatePath(myEEPROMDir, "eepromdir", myBaseDir);
+  validatePath(myStateDir, "statedir", myBaseDir + "state");
+  validatePath(mySnapshotSaveDir, "snapsavedir", defaultSnapSaveDir());
+  validatePath(mySnapshotLoadDir, "snaploaddir", defaultSnapLoadDir());
+  validatePath(myNVRamDir, "nvramdir", myBaseDir + "nvram");
   validatePath(myCfgDir, "cfgdir", myBaseDir + "cfg");
 
   s = mySettings->getString("cheatfile");
   if(s == "") s = myBaseDir + "stella.cht";
   node = FilesystemNode(s);
   myCheatFile = node.getPath();
-  mySettings->setString("cheatfile", node.getRelativePath());
+  mySettings->setValue("cheatfile", node.getShortPath());
 
   s = mySettings->getString("palettefile");
   if(s == "") s = myBaseDir + "stella.pal";
   node = FilesystemNode(s);
   myPaletteFile = node.getPath();
-  mySettings->setString("palettefile", node.getRelativePath());
+  mySettings->setValue("palettefile", node.getShortPath());
 
   s = mySettings->getString("propsfile");
   if(s == "") s = myBaseDir + "stella.pro";
   node = FilesystemNode(s);
   myPropertiesFile = node.getPath();
-  mySettings->setString("propsfile", node.getRelativePath());
+  mySettings->setValue("propsfile", node.getShortPath());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -380,12 +385,10 @@ void OSystem::setUIPalette()
 void OSystem::setBaseDir(const string& basedir)
 {
   FilesystemNode node(basedir);
-  myBaseDir = node.getPath();
   if(!node.isDirectory())
-  {
-    AbstractFilesystemNode::makeDir(myBaseDir);
-    myBaseDir = FilesystemNode(node.getPath()).getPath();
-  }
+    node.makeDir();
+
+  myBaseDir = node.getPath();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -448,7 +451,7 @@ FBInitStatus OSystem::createFrameBuffer()
 #endif
 
     default:  // Should never happen
-      logMessage("ERROR: Unknown emulation state in createFrameBuffer()\n", 0);
+      logMessage("ERROR: Unknown emulation state in createFrameBuffer()", 0);
       break;
   }
 
@@ -472,9 +475,9 @@ fallback:
   if(fbstatus == kFailNotSupported && myFrameBuffer &&
      myFrameBuffer->type() == kDoubleBuffer)
   {
-    logMessage("ERROR: OpenGL mode failed, fallback to software\n", 0);
+    logMessage("ERROR: OpenGL mode failed, fallback to software", 0);
     delete myFrameBuffer; myFrameBuffer = NULL;
-    mySettings->setString("video", "soft");
+    mySettings->setValue("video", "soft");
     FBInitStatus newstatus = createFrameBuffer();
     if(newstatus == kSuccess)
     {
@@ -494,46 +497,55 @@ void OSystem::createSound()
   if(!mySound)
     mySound = MediaFactory::createAudio(this);
 #ifndef SOUND_SUPPORT
-  mySettings->setBool("sound", false);
+  mySettings->setValue("sound", false);
 #endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::createConsole(const string& romfile, const string& md5sum)
+string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
+                              bool newrom)
 {
-  ostringstream buf;
-
   // Do a little error checking; it shouldn't be necessary
   if(myConsole) deleteConsole();
 
   bool showmessage = false;
 
-  // If a blank ROM has been given, we reload the current one (assuming one exists)
-  if(romfile == "")
+  // If same ROM has been given, we reload the current one (assuming one exists)
+  if(!newrom && rom == myRomFile)
   {
     showmessage = true;  // we show a message if a ROM is being reloaded
-    if(myRomFile == "")
-    {
-      logMessage("ERROR: Rom file not specified ...\n", 0);
-      return false;
-    }
   }
   else
   {
-    myRomFile = romfile;
+    myRomFile = rom;
     myRomMD5  = md5sum;
 
     // Each time a new console is loaded, we simulate a cart removal
     // Some carts need knowledge of this, as they behave differently
     // based on how many power-cycles they've been through since plugged in
-    mySettings->setInt("romloadcount", 0);
+    mySettings->setValue("romloadcount", 0);
   }
 
   // Create an instance of the 2600 game console
+  ostringstream buf;
   string type, id;
-  myConsole = openConsole(myRomFile, myRomMD5, type, id);
+  try
+  {
+    myConsole = openConsole(myRomFile, myRomMD5, type, id);
+  }
+  catch(const char* err_msg)
+  {
+    myConsole = 0;
+    buf << "ERROR: Couldn't create console (" << err_msg << ")";
+    logMessage(buf.str(), 0);
+    return buf.str();
+  }
+
   if(myConsole)
   {
+  #ifdef DEBUGGER_SUPPORT
+    myConsole->addDebugger();
+  #endif
   #ifdef CHEATCODE_SUPPORT
     myCheatManager->loadCheats(myRomMD5);
   #endif
@@ -551,9 +563,9 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
     myEventHandler->setMouseControllerMode(mySettings->getBool("usemouse"));
     if(createFrameBuffer() != kSuccess)  // Takes care of initializeVideo()
     {
-      logMessage("ERROR: Couldn't create framebuffer for console\n", 0);
+      logMessage("ERROR: Couldn't create framebuffer for console", 0);
       myEventHandler->reset(EventHandler::S_LAUNCHER);
-      return false;
+      return "ERROR: Couldn't create framebuffer for console";
     }
 
     if(showmessage)
@@ -564,7 +576,7 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
         myFrameBuffer->showMessage("Multicart " + type + ", loading ROM" + id);
     }
     buf << "Game console created:" << endl
-        << "  ROM file: " << FilesystemNode(myRomFile).getRelativePath() << endl << endl
+        << "  ROM file: " << myRomFile.getShortPath() << endl << endl
         << getROMInfo(myConsole) << endl;
     logMessage(buf.str(), 1);
 
@@ -581,15 +593,8 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
       myEventHandler->handleEvent(Event::ConsoleSelect, 1);
     if(mySettings->getBool("holdbutton0"))
       myEventHandler->handleEvent(Event::JoystickZeroFire, 1);
-
-    return true;
-  }
-  else
-  {
-    buf << "ERROR: Couldn't create console for " << myRomFile << endl;
-    logMessage(buf.str(), 0);
-    return false;
   }
+  return EmptyString;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -612,14 +617,23 @@ void OSystem::deleteConsole()
     logMessage(buf.str(), 1);
 
     delete myConsole;  myConsole = NULL;
+  #ifdef DEBUGGER_SUPPORT
     delete myDebugger; myDebugger = NULL;
+  #endif
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool OSystem::reloadConsole()
+{
+  deleteConsole();
+  return createConsole(myRomFile, myRomMD5, false) == EmptyString;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool OSystem::createLauncher(const string& startdir)
 {
-  mySettings->setString("tmpromdir", startdir);
+  mySettings->setValue("tmpromdir", startdir);
   bool status = false;
 
   myEventHandler->reset(EventHandler::S_LAUNCHER);
@@ -634,60 +648,51 @@ bool OSystem::createLauncher(const string& startdir)
     status = true;
   }
   else
-    logMessage("ERROR: Couldn't create launcher\n", 0);
+    logMessage("ERROR: Couldn't create launcher", 0);
 
   myLauncherUsed = myLauncherUsed || status;
   return status;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string OSystem::getROMInfo(const string& romfile)
+string OSystem::getROMInfo(const FilesystemNode& romfile)
 {
   string md5, type, id, result = "";
-  Console* console = openConsole(romfile, md5, type, id);
-  if(console)
+  Console* console = 0;
+  try
   {
-    result = getROMInfo(console);
-    delete console;
+    console = openConsole(romfile, md5, type, id);
+  }
+  catch(const char* err_msg)
+  {
+    ostringstream buf;
+    buf << "ERROR: Couldn't get ROM info (" << err_msg << ")";
+    return buf.str();
   }
-  else
-    result = "ERROR: Couldn't get ROM info for " + romfile + " ...";
 
+  result = getROMInfo(console);
+  delete console;
   return result;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string OSystem::MD5FromFile(const string& filename)
-{
-  string md5 = "";
-
-  uInt8* image = 0;
-  uInt32 size  = 0;
-  if((image = openROM(filename, md5, size)) != 0)
-    if(image != 0 && size > 0)
-      delete[] image;
-
-  return md5;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::logMessage(const string& message, uInt8 level)
 {
   if(level == 0)
   {
-    cout << message << flush;
-    myLogMessages += message;
+    cout << message << endl << flush;
+    myLogMessages += message + "\n";
   }
   else if(level <= (uInt8)mySettings->getInt("loglevel"))
   {
     if(mySettings->getBool("logtoconsole"))
-      cout << message << flush;
-    myLogMessages += message;
+      cout << message << endl << flush;
+    myLogMessages += message + "\n";
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Console* OSystem::openConsole(const string& romfile, string& md5,
+Console* OSystem::openConsole(const FilesystemNode& romfile, string& md5,
                               string& type, string& id)
 {
 #define CMDLINE_PROPS_UPDATE(cl_name, prop_name) \
@@ -719,12 +724,11 @@ Console* OSystem::openConsole(const string& romfile, string& md5,
     // and that the md5 (and hence the cart) has changed
     if(props.get(Cartridge_MD5) != cartmd5)
     {
-      string name = props.get(Cartridge_Name);
       if(!myPropSet->getMD5(cartmd5, props))
       {
         // Cart md5 wasn't found, so we create a new props for it
         props.set(Cartridge_MD5, cartmd5);
-        props.set(Cartridge_Name, name+id);
+        props.set(Cartridge_Name, props.get(Cartridge_Name)+id);
         myPropSet->insert(props, false);
       }
     }
@@ -750,12 +754,6 @@ Console* OSystem::openConsole(const string& romfile, string& md5,
     if(cart)
       console = new Console(this, cart, props);
   }
-  else
-  {
-    ostringstream buf;
-    buf << "ERROR: Couldn't open \'" << romfile << "\'" << endl;
-    logMessage(buf.str(), 0);
-  }
 
   // Free the image since we don't need it any longer
   if(image != 0 && size > 0)
@@ -765,7 +763,7 @@ Console* OSystem::openConsole(const string& romfile, string& md5,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
+uInt8* OSystem::openROM(const FilesystemNode& rom, string& md5, uInt32& size)
 {
   // This method has a documented side-effect:
   // It not only loads a ROM and creates an array with its contents,
@@ -773,29 +771,7 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
   // contain a valid name
 
   uInt8* image = 0;
-
-  // First try to load as ZIP archive
-  if(loadFromZIP(file, &image, size))
-  {
-    // Empty file means it *was* a ZIP file, but it didn't contain
-    // a valid ROM
-    if(image == 0)
-      return image;
-  }
-  else
-  {
-    // Assume the file is either gzip'ed or not compressed at all
-    gzFile f = gzopen(file.c_str(), "rb");
-    if(!f)
-      return image;
-
-    image = new uInt8[MAX_ROM_SIZE];
-    size = gzread(f, image, MAX_ROM_SIZE);
-    gzclose(f);
-  }
-
-  // Zero-byte files should be automatically discarded
-  if(size == 0)
+  if((size = rom.read(image)) == 0)
   {
     delete[] image;
     return (uInt8*) 0;
@@ -811,104 +787,11 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
   // be an entry in stella.pro.  In that case, we use the rom name
   // and reinsert the properties object
   Properties props;
-  if(!myPropSet->getMD5(md5, props))
-  {
-    // Get the filename from the rom pathname
-    FilesystemNode node(file);
-    file = node.getDisplayName();
-
-    props.set(Cartridge_MD5, md5);
-    props.set(Cartridge_Name, file);
-    myPropSet->insert(props, false);
-  }
+  myPropSet->getMD5WithInsert(rom, md5, props);
 
   return image;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::loadFromZIP(const string& filename, uInt8** image, uInt32& size)
-{
-  // First determine if this actually is a ZIP file
-  // by seeing if it contains the '.zip' extension
-  size_t extpos = BSPF_findIgnoreCase(filename, ".zip");
-  if(extpos == string::npos)
-    return false;
-
-  // Now get the file after the .zip extension (if any)
-  string archive = filename, fileinzip = "";
-  if(filename.size() > extpos + 4)
-  {
-    archive = filename.substr(0, extpos + 4);
-    fileinzip = filename.substr(extpos + 5);
-    if(fileinzip.size() == 0)
-      return true;  // This was a ZIP file, but invalid name
-  }
-
-  // Open archive
-  unzFile tz;
-  if((tz = unzOpen(archive.c_str())) != NULL)
-  {
-    if(unzGoToFirstFile(tz) == UNZ_OK)
-    {
-      unz_file_info ufo;
-
-      for(;;)  // Loop through all files for valid 2600 images
-      {
-        // Longer filenames might be possible, but I don't
-        // think people would name files that long in zip files...
-        char currfile[1024];
-
-        unzGetCurrentFileInfo(tz, &ufo, currfile, 1024, 0, 0, 0, 0);
-        currfile[1023] = '\0';
-
-        if(strlen(currfile) >= 4 &&
-           !BSPF_startsWithIgnoreCase(filename, "__MACOSX"))
-        {
-          // Grab 3-character extension
-          const char* ext = currfile + strlen(currfile) - 4;
-
-          if(BSPF_equalsIgnoreCase(ext, ".a26") || BSPF_equalsIgnoreCase(ext, ".bin") ||
-             BSPF_equalsIgnoreCase(ext, ".rom"))
-          {
-            // Either match the first file or the one we're looking for
-            if(fileinzip.empty() || fileinzip == currfile)
-              break;
-          }
-        }
-
-        // Scan the next file in the zip
-        if(unzGoToNextFile(tz) != UNZ_OK)
-          break;
-      }
-
-      // Now see if we got a valid image
-      if(ufo.uncompressed_size <= 0)
-      {
-        unzClose(tz);
-        return true;
-      }
-      size  = ufo.uncompressed_size;
-      *image = new uInt8[size];
-
-      // We don't have to check for any return errors from these functions,
-      // since if there are, 'image' will not contain a valid ROM and the
-      // calling method can take care of it
-      unzOpenCurrentFile(tz);
-      unzReadCurrentFile(tz, *image, size);
-      unzCloseCurrentFile(tz);
-      unzClose(tz);
-
-      return true;
-    }
-    else
-    {
-      unzClose(tz);
-      return true;
-    }
-  }
-  return false;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string OSystem::getROMInfo(const Console* console)
 {
@@ -942,12 +825,10 @@ void OSystem::validatePath(string& path, const string& setting,
                     mySettings->getString(setting);
   FilesystemNode node(s);
   if(!node.isDirectory())
-  {
-    AbstractFilesystemNode::makeDir(s);
-    node = FilesystemNode(node.getPath());
-  }
+    node.makeDir();
+
   path = node.getPath();
-  mySettings->setString(setting, node.getRelativePath());
+  mySettings->setValue(setting, node.getShortPath());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1019,16 +900,6 @@ void OSystem::setDefaultJoymap(Event::Type event, EventMode mode)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::pollEvent()
-{
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::stateChanged(EventHandler::State state)
-{
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt64 OSystem::getTicks() const
 {
@@ -1110,9 +981,8 @@ bool OSystem::queryVideoHardware()
   // Check the 'maxres' setting, which is an undocumented developer feature
   // that specifies the desktop size
   // Normally, this wouldn't be set, and we ask SDL directly
-  int w, h;
-  mySettings->getSize("maxres", w, h);
-  if(w <= 0 || h <= 0)
+  const GUI::Size& s = mySettings->getSize("maxres");
+  if(s.w <= 0 || s.h <= 0)
   {
     const SDL_VideoInfo* info = SDL_GetVideoInfo();
     myDesktopWidth  = info->current_w;
@@ -1120,12 +990,17 @@ bool OSystem::queryVideoHardware()
   }
   else
   {
-    myDesktopWidth  = BSPF_max(w, 320);
-    myDesktopHeight = BSPF_max(h, 240);
+    myDesktopWidth  = BSPF_max(s.w, 320);
+    myDesktopHeight = BSPF_max(s.h, 240);
   }
 
   // Various parts of the codebase assume a minimum screen size of 320x240
-  assert(myDesktopWidth >= 320 && myDesktopHeight >= 240);
+  if(!(myDesktopWidth >= 320 && myDesktopHeight >= 240))
+  {
+    logMessage("ERROR: queryVideoHardware failed, "
+               "window 320x240 or larger required", 0);
+    return false;
+  }
 
   // Then get the valid fullscreen modes
   // If there are any errors, just use the desktop resolution
@@ -1244,3 +1119,6 @@ OSystem& OSystem::operator = (const OSystem&)
   assert(false);
   return *this;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ZipHandler* OSystem::myZipHandler = 0;
