@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PropsSet.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: PropsSet.cxx 2753 2013-06-21 12:15:32Z stephena $
 //============================================================================
 
 #include <fstream>
@@ -32,8 +32,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PropertiesSet::PropertiesSet(OSystem* osystem)
-  : myOSystem(osystem),
-    mySize(0)
+  : myOSystem(osystem)
 {
   load(myOSystem->propertiesFile());
 }
@@ -148,6 +147,20 @@ bool PropertiesSet::getMD5(const string& md5, Properties& properties,
   return found;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PropertiesSet::getMD5WithInsert(const FilesystemNode& rom,
+                                     const string& md5, Properties& properties)
+{
+  if(!getMD5(md5, properties))
+  {
+    properties.set(Cartridge_MD5, md5);
+    // Create a name suitable for using in properties
+    properties.set(Cartridge_Name, rom.getNameWithExt(""));
+
+    insert(properties, false);
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PropertiesSet::insert(const Properties& properties, bool save)
 {
