@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx 2504 2012-05-27 19:27:55Z stephena $
+// $Id: Settings.cxx 2756 2013-06-26 16:03:08Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -61,7 +61,7 @@ Settings::Settings(OSystem* osystem)
 
   // TV filtering options
   setInternal("tv_filter", "0");
-  setInternal("tv_scanlines", "40");
+  setInternal("tv_scanlines", "25");
   setInternal("tv_scaninter", "true");
   // TV options when using 'custom' mode
   setInternal("tv_contrast", "0.0");
@@ -79,9 +79,7 @@ Settings::Settings(OSystem* osystem)
   setInternal("sound", "true");
   setInternal("fragsize", "512");
   setInternal("freq", "31400");
-  setInternal("tiafreq", "31400");
   setInternal("volume", "100");
-  setInternal("clipvol", "true");
 
   // Input event options
   setInternal("keymap", "");
@@ -96,7 +94,9 @@ Settings::Settings(OSystem* osystem)
   setInternal("ctrlcombo", "true");
 
   // Snapshot options
-  setInternal("snapdir", "");
+  setInternal("snapsavedir", "");
+  setInternal("snaploaddir", "");
+  setInternal("snapname", "int");
   setInternal("sssingle", "false");
   setInternal("ss1x", "false");
   setInternal("ssinterval", "2");
@@ -107,7 +107,7 @@ Settings::Settings(OSystem* osystem)
   setInternal("cheatfile", "");
   setInternal("palettefile", "");
   setInternal("propsfile", "");
-  setInternal("eepromdir", "");
+  setInternal("nvramdir", "");
   setInternal("cfgdir", "");
 
   // ROM browser options
@@ -137,7 +137,7 @@ Settings::Settings(OSystem* osystem)
   setExternal("maxres", "");
 
   // Debugger disassembly options
-  setInternal("dis.resolvedata", "auto");
+  setInternal("dis.resolve", "true");
   setInternal("dis.gfxformat", "2");
   setInternal("dis.showaddr", "true");
   setInternal("dis.relocate", "false");
@@ -162,7 +162,7 @@ void Settings::loadConfig()
   ifstream in(myOSystem->configFile().c_str());
   if(!in || !in.is_open())
   {
-    myOSystem->logMessage("ERROR: Couldn't load settings file\n", 0);
+    myOSystem->logMessage("ERROR: Couldn't load settings file", 0);
     return;
   }
 
@@ -286,9 +286,8 @@ void Settings::validate()
   i = getInt("volume");
   if(i < 0 || i > 100)    setInternal("volume", "100");
   i = getInt("freq");
-  if(i < 0 || i > 48000)  setInternal("freq", "31400");
-  i = getInt("tiafreq");
-  if(i < 0 || i > 48000)  setInternal("tiafreq", "31400");
+  if(!(i == 11025 || i == 22050 || i == 31400 || i == 44100 || i == 48000))
+    setInternal("freq", "31400");
 #endif
 
   i = getInt("joydeadzone");
@@ -383,10 +382,8 @@ void Settings::usage()
   #ifdef SOUND_SUPPORT
     << "  -sound        <1|0>          Enable sound generation\n"
     << "  -fragsize     <number>       The size of sound fragments (must be a power of two)\n"
-    << "  -freq         <number>       Set sound sample output frequency (0 - 48000)\n"
-    << "  -tiafreq      <number>       Set sound sample generation frequency (0 - 48000)\n"
+    << "  -freq         <number>       Set sound sample output frequency (11025|22050|31400|44100|48000)\n"
     << "  -volume       <number>       Set the volume (0 - 100)\n"
-    << "  -clipvol      <1|0>          Enable volume clipping (eliminates popping)\n"
     << endl
   #endif
     << "  -cheat        <code>         Use the specified cheatcode (see manual for description)\n"
@@ -402,7 +399,9 @@ void Settings::usage()
     << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
     << "  -stats        <1|0>          Overlay console info during emulation\n"
     << "  -fastscbios   <1|0>          Disable Supercharger BIOS progress loading bars\n"
-    << "  -snapdir      <path>         The directory to save snapshot files to\n"
+    << "  -snapsavedir  <path>         The directory to save snapshot files to\n"
+    << "  -snaploaddir  <path>         The directory to load snapshot files from\n"
+    << "  -snapname     <int|rom>      Name snapshots according to internal database or ROM\n"
     << "  -sssingle     <1|0>          Generate single snapshot instead of many\n"
     << "  -ss1x         <1|0>          Generate TIA snapshot in 1x mode (ignore scaling/effects)\n"
     << "  -ssinterval   <number        Number of seconds between snapshots in continuous snapshot mode\n"
@@ -420,11 +419,11 @@ void Settings::usage()
     << "  -uipalette    <1|2>          Used the specified palette for UI elements\n"
     << "  -listdelay    <delay>        Time to wait between keypresses in list widgets (300-1000)\n"
     << "  -mwheel       <lines>        Number of lines the mouse wheel will scroll in UI\n"
-    << "  -statedir     <dir>          Directory in which to save state files\n"
+    << "  -statedir     <dir>          Directory in which to save/load state files\n"
     << "  -cheatfile    <file>         Full pathname of cheatfile database\n"
     << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
     << "  -propsfile    <file>         Full pathname of ROM properties file\n"
-    << "  -eepromdir    <dir>          Directory in which to save EEPROM files\n"
+    << "  -nvramdir     <dir>          Directory in which to save/load flash/EEPROM files\n"
     << "  -cfgdir       <dir>          Directory in which to save Distella config files\n"
     << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
     << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
@@ -439,12 +438,10 @@ void Settings::usage()
     << " The following options are meant for developers\n"
     << " Arguments are more fully explained in the manual\n"
     << endl
-    << "   -dis.resolvedata <never|    Set automatic code vs. data determination in disassembler\n"
-    << "                     always|\n"
-    << "                     auto>\n"
-    << "   -dis.gfxformat   <2|16>     Set base to use for displaying GFX sections in disassembler\n"
-    << "   -dis.showaddr    <1|0>      Show opcode addresses in disassembler\n"
-    << "   -dis.relocate    <1|0>      Relocate calls out of address range in disassembler\n"
+    << "   -dis.resolve   <1|0>        Attempt to resolve code sections in disassembler\n"
+    << "   -dis.gfxformat <2|16>       Set base to use for displaying GFX sections in disassembler\n"
+    << "   -dis.showaddr  <1|0>        Show opcode addresses in disassembler\n"
+    << "   -dis.relocate  <1|0>        Relocate calls out of address range in disassembler\n"
     << endl
     << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
     << "   -break        <address>     Set a breakpoint at 'address'\n"
@@ -470,6 +467,28 @@ void Settings::usage()
     << endl << flush;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const Variant& Settings::value(const string& key) const
+{
+  // Try to find the named setting and answer its value
+  int idx = -1;
+  if((idx = getInternalPos(key)) != -1)
+    return myInternalSettings[idx].value;
+  else if((idx = getExternalPos(key)) != -1)
+    return myExternalSettings[idx].value;
+  else
+    return EmptyVariant;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::setValue(const string& key, const Variant& value)
+{
+  if(int idx = getInternalPos(key) != -1)
+    setInternal(key, value, idx);
+  else
+    setExternal(key, value);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Settings::saveConfig()
 {
@@ -491,7 +510,7 @@ void Settings::saveConfig()
   ofstream out(myOSystem->configFile().c_str());
   if(!out || !out.is_open())
   {
-    myOSystem->logMessage("ERROR: Couldn't save settings file\n", 0);
+    myOSystem->logMessage("ERROR: Couldn't save settings file", 0);
     return;
   }
 
@@ -520,139 +539,6 @@ void Settings::saveConfig()
   out.close();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setInt(const string& key, const int value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setFloat(const string& key, const float value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setBool(const string& key, const bool value)
-{
-  ostringstream stream;
-  stream << value;
-
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, stream.str(), idx);
-  else
-    setExternal(key, stream.str());
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setString(const string& key, const string& value)
-{
-  if(int idx = getInternalPos(key) != -1)
-    setInternal(key, value, idx);
-  else
-    setExternal(key, value);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::getSize(const string& key, int& x, int& y) const
-{
-  char c = '\0';
-  x = y = -1;
-  string size = getString(key);
-  istringstream buf(size);
-  buf >> x >> c >> y;
-  if(c != 'x')
-    x = y = -1;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::getInt(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return (int) atoi(myInternalSettings[idx].value.c_str());
-  else if((idx = getExternalPos(key)) != -1)
-    return (int) atoi(myExternalSettings[idx].value.c_str());
-  else
-    return -1;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-float Settings::getFloat(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return (float) atof(myInternalSettings[idx].value.c_str());
-  else if((idx = getExternalPos(key)) != -1)
-    return (float) atof(myExternalSettings[idx].value.c_str());
-  else
-    return -1.0;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Settings::getBool(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-  {
-    const string& value = myInternalSettings[idx].value;
-    if(value == "1" || value == "true")
-      return true;
-    else if(value == "0" || value == "false")
-      return false;
-    else
-      return false;
-  }
-  else if((idx = getExternalPos(key)) != -1)
-  {
-    const string& value = myExternalSettings[idx].value;
-    if(value == "1" || value == "true")
-      return true;
-    else if(value == "0" || value == "false")
-      return false;
-    else
-      return false;
-  }
-  else
-    return false;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& Settings::getString(const string& key) const
-{
-  // Try to find the named setting and answer its value
-  int idx = -1;
-  if((idx = getInternalPos(key)) != -1)
-    return myInternalSettings[idx].value;
-  else if((idx = getExternalPos(key)) != -1)
-    return myExternalSettings[idx].value;
-  else
-    return EmptyString;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Settings::setSize(const string& key, const int value1, const int value2)
-{
-  ostringstream buf;
-  buf << value1 << "x" << value2;
-  setString(key, buf.str());
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInternalPos(const string& key) const
 {
@@ -674,7 +560,7 @@ int Settings::getExternalPos(const string& key) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::setInternal(const string& key, const string& value,
+int Settings::setInternal(const string& key, const Variant& value,
                           int pos, bool useAsInitial)
 {
   int idx = -1;
@@ -729,7 +615,7 @@ int Settings::setInternal(const string& key, const string& value,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Settings::setExternal(const string& key, const string& value,
+int Settings::setExternal(const string& key, const Variant& value,
                           int pos, bool useAsInitial)
 {
   int idx = -1;
