@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: StateManager.cxx 2527 2012-06-05 16:32:35Z stephena $
+// $Id: StateManager.cxx 2725 2013-05-07 18:53:21Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -30,15 +30,14 @@
 
 #include "StateManager.hxx"
 
-#define STATE_HEADER "03070100state"
+#define STATE_HEADER "03070300state"
 #define MOVIE_HEADER "03030000movie"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 StateManager::StateManager(OSystem* osystem)
   : myOSystem(osystem),
     myCurrentSlot(0),
-    myActiveMode(kOffMode),
-    myFrameCounter(0)
+    myActiveMode(kOffMode)
 {
   reset();
 }
