@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIA.cxx 2527 2012-06-05 16:32:35Z stephena $
+// $Id: TIA.cxx 2640 2013-02-28 14:39:00Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -39,7 +39,8 @@
 #include "TIA.hxx"
 
 #define HBLANK 68
-#define USE_MMR_LATCHES
+
+#define CLAMP_POS(reg) if(reg < 0) { reg += 160; }  reg %= 160;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TIA::TIA(Console& console, Sound& sound, Settings& settings)
@@ -141,12 +142,12 @@ void TIA::reset()
   myFrameCounter = myPALFrameCounter = 0;
   myScanlineCountForLastFrame = 0;
 
-  myCurrentP0Mask = &TIATables::PxMask[0][0][0][0];
-  myCurrentP1Mask = &TIATables::PxMask[0][0][0][0];
-  myCurrentM0Mask = &TIATables::MxMask[0][0][0][0];
-  myCurrentM1Mask = &TIATables::MxMask[0][0][0][0];
-  myCurrentBLMask = &TIATables::BLMask[0][0][0];
-  myCurrentPFMask = TIATables::PFMask[0];
+  myP0Mask = &TIATables::PxMask[0][0][0][0];
+  myP1Mask = &TIATables::PxMask[0][0][0][0];
+  myM0Mask = &TIATables::MxMask[0][0][0][0];
+  myM1Mask = &TIATables::MxMask[0][0][0][0];
+  myBLMask = &TIATables::BLMask[0][0][0];
+  myPFMask = TIATables::PFMask[0];
 
   // Recalculate the size of the display
   toggleFixedColors(0);
@@ -600,10 +601,54 @@ inline void TIA::startFrame()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline void TIA::endFrame()
 {
-  // This stuff should only happen at the end of a frame
+  uInt32 currentlines = scanlines();
+
+  // The TIA may generate frames that are 'invisible' to TV (they complete
+  // before the first visible scanline)
+  // Such 'short' frames can't simply be eliminated, since they're running
+  // code at that point; however, they are not shown at all, otherwise the
+  // double-buffering of the video output will get confused
+  if(currentlines <= myStartScanline)
+  {
+    // Skip display of this frame, as if it wasn't generated at all
+    startFrame();
+    return;
+  }
+
   // Compute the number of scanlines in the frame
   uInt32 previousCount = myScanlineCountForLastFrame;
-  myScanlineCountForLastFrame = scanlines();
+  myScanlineCountForLastFrame = currentlines;
+
+  // The following handle cases where scanlines either go too high or too
+  // low compared to the previous frame, in which case certain portions
+  // of the framebuffer are cleared to zero (black pixels)
+  // Due to the FrameBuffer class (potentially) doing dirty-rectangle
+  // updates, each internal buffer must be set slightly differently,
+  // otherwise they won't know anything has changed
+  // Hence, the front buffer is set to pixel 0, and the back to pixel 1
+
+  // Did we generate too many scanlines?
+  // (usually caused by VBLANK/VSYNC taking too long or not occurring at all)
+  // If so, blank entire viewable area
+  if(myScanlineCountForLastFrame > myMaximumNumberOfScanlines+1)
+  {
+    myScanlineCountForLastFrame = myMaximumNumberOfScanlines;
+    if(previousCount < myMaximumNumberOfScanlines)
+    {
+      memset(myCurrentFrameBuffer, 0, 160 * 320);
+      memset(myPreviousFrameBuffer, 1, 160 * 320);
+    }
+  }
+  // Did the number of scanlines decrease?
+  // If so, blank scanlines that weren't rendered this frame
+  else if(myScanlineCountForLastFrame < previousCount &&
+          myScanlineCountForLastFrame < 320 && previousCount < 320)
+  {
+    uInt32 offset = myScanlineCountForLastFrame * 160,
+           stride = (previousCount - myScanlineCountForLastFrame) * 160;
+    memset(myCurrentFrameBuffer + offset, 0, stride);
+    memset(myPreviousFrameBuffer + offset, 1, stride);
+  }
 
   // Stats counters
   myFrameCounter++;
@@ -611,8 +656,7 @@ inline void TIA::endFrame()
     myPALFrameCounter++;
 
   // Recalculate framerate. attempting to auto-correct for scanline 'jumps'
-  if(myFrameCounter % 8 == 0 && myAutoFrameEnabled &&
-     myScanlineCountForLastFrame < myMaximumNumberOfScanlines)
+  if(myAutoFrameEnabled)
   {
     myFramerate = (myScanlineCountForLastFrame > 285 ? 15600.0 : 15720.0) /
                    myScanlineCountForLastFrame;
@@ -625,33 +669,6 @@ inline void TIA::endFrame()
     if(offset > myStopDisplayOffset && offset < 228 * 320)
       myStopDisplayOffset = offset;
   }
-
-  // The following handle cases where scanlines either go too high or too
-  // low compared to the previous frame, in which case certain portions
-  // of the framebuffer are cleared to zero (black pixels)
-  // Due to the FrameBuffer class (potentially) doing dirty-rectangle
-  // updates, each internal buffer must be set slightly differently,
-  // otherwise they won't know anything has changed
-  // Hence, the front buffer is set to pixel 0, and the back to pixel 1
-
-  // Did we generate too many scanlines?
-  // (usually caused by VBLANK taking too long)
-  // If so, blank entire viewable area
-  if(myScanlineCountForLastFrame > 342 && previousCount <= 342)
-  {
-    memset(myCurrentFrameBuffer, 0, 160 * 320);
-    memset(myPreviousFrameBuffer, 1, 160 * 320);
-  }
-  // Did the number of scanlines decrease?
-  // If so, blank scanlines that weren't rendered this frame
-  else if(myScanlineCountForLastFrame < previousCount &&
-          myScanlineCountForLastFrame < 320 && previousCount < 320)
-  {
-    uInt32 offset = myScanlineCountForLastFrame * 160,
-           stride = (previousCount - myScanlineCountForLastFrame) * 160;
-    memset(myCurrentFrameBuffer + offset, 0, stride);
-    memset(myPreviousFrameBuffer + offset, 1, stride);
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -851,9 +868,9 @@ void TIA::updateScanline()
 
   int clock;
   do {
-	  mySystem->m6502().execute(1);
-	  clock = mySystem->cycles() * 3;
-	  updateFrame(clock);
+    mySystem->m6502().execute(1);
+    clock = mySystem->cycles() * 3;
+    updateFrame(clock);
   } while(clock < endClock);
 
   // if we finished the frame, get ready for the next one
@@ -937,33 +954,32 @@ void TIA::updateFrame(Int32 clock)
       {
         if(myCurrentHMOVEPos >= 97 && myCurrentHMOVEPos < 157)
         {
-          myPOSP0 -= myMotionClockP0;
-          myPOSP1 -= myMotionClockP1;
-          myPOSM0 -= myMotionClockM0;
-          myPOSM1 -= myMotionClockM1;
-          myPOSBL -= myMotionClockBL;
+          myPOSP0 -= myMotionClockP0;  if(myPOSP0 < 0) myPOSP0 += 160;
+          myPOSP1 -= myMotionClockP1;  if(myPOSP1 < 0) myPOSP1 += 160;
+          myPOSM0 -= myMotionClockM0;  if(myPOSM0 < 0) myPOSM0 += 160;
+          myPOSM1 -= myMotionClockM1;  if(myPOSM1 < 0) myPOSM1 += 160;
+          myPOSBL -= myMotionClockBL;  if(myPOSBL < 0) myPOSBL += 160;
+
           myPreviousHMOVEPos = myCurrentHMOVEPos;
-          posChanged = true;
         }
         // Indicate that the HMOVE has been completed
         myCurrentHMOVEPos = 0x7FFFFFFF;
+        posChanged = true;
       }
-#ifdef USE_MMR_LATCHES
+
       // Apply extra clocks for 'more motion required/mmr'
-      if(myHMP0mmr) { myPOSP0 -= 17; posChanged = true; }
-      if(myHMP1mmr) { myPOSP1 -= 17; posChanged = true; }
-      if(myHMM0mmr) { myPOSM0 -= 17; posChanged = true; }
-      if(myHMM1mmr) { myPOSM1 -= 17; posChanged = true; }
-      if(myHMBLmmr) { myPOSBL -= 17; posChanged = true; }
-#endif
-      // Make sure positions are in range
+      if(myHMP0mmr) { myPOSP0 -= 17;  if(myPOSP0 < 0) myPOSP0 += 160;  posChanged = true; }
+      if(myHMP1mmr) { myPOSP1 -= 17;  if(myPOSP1 < 0) myPOSP1 += 160;  posChanged = true; }
+      if(myHMM0mmr) { myPOSM0 -= 17;  if(myPOSM0 < 0) myPOSM0 += 160;  posChanged = true; }
+      if(myHMM1mmr) { myPOSM1 -= 17;  if(myPOSM1 < 0) myPOSM1 += 160;  posChanged = true; }
+      if(myHMBLmmr) { myPOSBL -= 17;  if(myPOSBL < 0) myPOSBL += 160;  posChanged = true; }
+
+      // Scanline change, so reset PF mask based on current CTRLPF reflection state 
+      myPFMask = TIATables::PFMask[myCTRLPF & 0x01];
+
+      // TODO - handle changes to player timing
       if(posChanged)
       {
-        if(myPOSP0 < 0) { myPOSP0 += 160; }  myPOSP0 %= 160;
-        if(myPOSP1 < 0) { myPOSP1 += 160; }  myPOSP1 %= 160;
-        if(myPOSM0 < 0) { myPOSM0 += 160; }  myPOSM0 %= 160;
-        if(myPOSM1 < 0) { myPOSM1 += 160; }  myPOSM1 %= 160;
-        if(myPOSBL < 0) { myPOSBL += 160; }  myPOSBL %= 160;
       }
     }
 
@@ -1024,12 +1040,12 @@ void TIA::updateFrame(Int32 clock)
       else
       {
         // Update masks
-        myCurrentBLMask = &TIATables::BLMask[myPOSBL & 0x03]
-            [(myCTRLPF & 0x30) >> 4][160 - (myPOSBL & 0xFC)];
-        myCurrentP0Mask = &TIATables::PxMask[myPOSP0 & 0x03]
+        myP0Mask = &TIATables::PxMask[myPOSP0 & 0x03]
             [mySuppressP0][myNUSIZ0 & 0x07][160 - (myPOSP0 & 0xFC)];
-        myCurrentP1Mask = &TIATables::PxMask[myPOSP1 & 0x03]
+        myP1Mask = &TIATables::PxMask[myPOSP1 & 0x03]
             [mySuppressP1][myNUSIZ1 & 0x07][160 - (myPOSP1 & 0xFC)];
+        myBLMask = &TIATables::BLMask[myPOSBL & 0x03]
+            [(myCTRLPF & 0x30) >> 4][160 - (myPOSBL & 0xFC)];
 
         // TODO - 08-27-2009: Simulate the weird effects of Cosmic Ark and
         // Stay Frosty.  The movement itself is well understood, but there
@@ -1045,21 +1061,23 @@ void TIA::updateFrame(Int32 clock)
             case 3:
               // Stretch this missle so it's 2 pixels wide and shifted one
               // pixel to the left
-              myCurrentM0Mask = &TIATables::MxMask[(myPOSM0-1) & 0x03]
-                  [myNUSIZ0 & 0x07][((myNUSIZ0 & 0x30) >> 4)|1][160 - ((myPOSM0-1) & 0xFC)];
+              myM0Mask = &TIATables::MxMask[(myPOSM0-1) & 0x03]
+                  [myNUSIZ0 & 0x07][((myNUSIZ0 & 0x30) >> 4)|1]
+                  [160 - ((myPOSM0-1) & 0xFC)];
               break;
             case 2:
               // Missle is disabled on this line
-              myCurrentM0Mask = &TIATables::DisabledMask[0];
+              myM0Mask = &TIATables::DisabledMask[0];
               break;
             default:
-              myCurrentM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
-                  [myNUSIZ0 & 0x07][(myNUSIZ0 & 0x30) >> 4][160 - (myPOSM0 & 0xFC)];
+              myM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
+                  [myNUSIZ0 & 0x07][(myNUSIZ0 & 0x30) >> 4]
+                  [160 - (myPOSM0 & 0xFC)];
               break;
           }
         }
         else
-          myCurrentM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
+          myM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
               [myNUSIZ0 & 0x07][(myNUSIZ0 & 0x30) >> 4][160 - (myPOSM0 & 0xFC)];
         if(myHMM1mmr)
         {
@@ -1068,21 +1086,23 @@ void TIA::updateFrame(Int32 clock)
             case 3:
               // Stretch this missle so it's 2 pixels wide and shifted one
               // pixel to the left
-              myCurrentM1Mask = &TIATables::MxMask[(myPOSM1-1) & 0x03]
-                  [myNUSIZ1 & 0x07][((myNUSIZ1 & 0x30) >> 4)|1][160 - ((myPOSM1-1) & 0xFC)];
+              myM1Mask = &TIATables::MxMask[(myPOSM1-1) & 0x03]
+                  [myNUSIZ1 & 0x07][((myNUSIZ1 & 0x30) >> 4)|1]
+                  [160 - ((myPOSM1-1) & 0xFC)];
               break;
             case 2:
               // Missle is disabled on this line
-              myCurrentM1Mask = &TIATables::DisabledMask[0];
+              myM1Mask = &TIATables::DisabledMask[0];
               break;
             default:
-              myCurrentM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
-                  [myNUSIZ1 & 0x07][(myNUSIZ1 & 0x30) >> 4][160 - (myPOSM1 & 0xFC)];
+              myM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
+                  [myNUSIZ1 & 0x07][(myNUSIZ1 & 0x30) >> 4]
+                  [160 - (myPOSM1 & 0xFC)];
               break;
           }
         }
         else
-          myCurrentM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
+          myM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
               [myNUSIZ1 & 0x07][(myNUSIZ1 & 0x30) >> 4][160 - (myPOSM1 & 0xFC)];
 
         uInt8 enabledObjects = myEnabledObjects & myDisabledObjects;
@@ -1090,21 +1110,21 @@ void TIA::updateFrame(Int32 clock)
         for(; myFramePointer < ending; ++myFramePointer, ++hpos)
         {
           uInt8 enabled = ((enabledObjects & PFBit) &&
-                           (myPF & myCurrentPFMask[hpos])) ? PFBit : 0;
+                           (myPF & myPFMask[hpos])) ? PFBit : 0;
 
-          if((enabledObjects & BLBit) && myCurrentBLMask[hpos])
+          if((enabledObjects & BLBit) && myBLMask[hpos])
             enabled |= BLBit;
 
-          if((enabledObjects & P1Bit) && (myCurrentGRP1 & myCurrentP1Mask[hpos]))
+          if((enabledObjects & P1Bit) && (myCurrentGRP1 & myP1Mask[hpos]))
             enabled |= P1Bit;
 
-          if((enabledObjects & M1Bit) && myCurrentM1Mask[hpos])
+          if((enabledObjects & M1Bit) && myM1Mask[hpos])
             enabled |= M1Bit;
 
-          if((enabledObjects & P0Bit) && (myCurrentGRP0 & myCurrentP0Mask[hpos]))
+          if((enabledObjects & P0Bit) && (myCurrentGRP0 & myP0Mask[hpos]))
             enabled |= P0Bit;
 
-          if((enabledObjects & M0Bit) && myCurrentM0Mask[hpos])
+          if((enabledObjects & M0Bit) && myM0Mask[hpos])
             enabled |= M0Bit;
 
           myCollision |= TIATables::CollisionMask[enabled];
@@ -1126,17 +1146,17 @@ void TIA::updateFrame(Int32 clock)
         myHMOVEBlankEnabled = false;
     }
 
+// TODO - this needs to be updated to actually do as the comment suggests
+#if 1
     // See if we're at the end of a scanline
     if(myClocksToEndOfScanLine == 228)
     {
-      // Yes, so set PF mask based on current CTRLPF reflection state 
-      myCurrentPFMask = TIATables::PFMask[myCTRLPF & 0x01];
-
       // TODO - 01-21-99: These should be reset right after the first copy
       // of the player has passed.  However, for now we'll just reset at the
       // end of the scanline since the other way would be too slow.
       mySuppressP0 = mySuppressP1 = 0;
     }
+#endif
   }
 }
 
@@ -1150,6 +1170,62 @@ inline void TIA::waitHorizontalSync()
     mySystem->incrementCycles(cyclesToEndOfLine);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+inline void TIA::waitHorizontalRSync()
+{
+  // 02-23-2013: RSYNC has now been updated to work correctly with
+  // Extra-Terrestrials. Fatal Run also uses RSYNC (in its VSYNC routine),
+  // and the NTSC prototype now displays 262 scanlines instead of 261.
+  // What is not emulated correctly is the "real time" effects. For example
+  // the VSYNC signal may not be 3 complete scanlines, although Stella will
+  // now count it as such.
+  //
+  // There are two extreme cases to demonstrate this "real time" variance
+  // effect over a proper three line VSYNC. 3*76 = 228 cycles properly needed:
+  //
+  // ======  SHORT TIME CASE  ======
+  // 
+  //     lda    #3      ;2  @67
+  //     sta    VSYNC   ;3  @70      vsync starts
+  //     sta    RSYNC   ;3  @73  +3
+  //     sta    WSYNC   ;3  @76  +6
+  // ------------------------------
+  //     sta    WSYNC   ;3  @76  +82
+  // ------------------------------
+  //     lda    #0      ;2  @2   +84
+  //     sta    VSYNC                vsync ends
+  //
+  // ======  LONG TIME CASE  ======
+  //
+  //    lda    #3      ;2  @70
+  //    sta    VSYNC   ;3  @73      vsync starts
+  //    sta    RSYNC   ;3  @74  +3
+  //    sta    WSYNC   ;3  @..  +81  2 cycles are added to previous line, and then
+  //                                 WSYNC halts the new line delaying 78 cycles total!
+  //------------------------------
+  //    sta    WSYNC   ;3  @76  +157
+  //------------------------------
+  //    lda    #0      ;2  @2   +159
+  //    sta    VSYNC                vsync ends
+
+  // The significance of the 'magic numbers' below is as follows (thanks to
+  // Eckhard Stolberg and Omegamatrix for explanation and implementation)
+  //
+  // Objects always get positioned three pixels further to the right after a
+  // WSYNC than they do after a RSYNC, but this is to be expected.  Triggering
+  // WSYNC will halt the CPU until the horizontal sync counter wraps around to zero.
+  // Triggering RSYNC will reset the horizontal sync counter to zero immediately.
+  // But the warp-around will actually happen after one more cycle of this counter.
+  // Since the horizontal sync counter counts once every 4 pixels, one more CPU
+  // cycle occurs before the counter warps around to zero. Therefore the positioning
+  // code will hit RESPx one cycle sooner after a RSYNC than after a WSYNC.
+
+  uInt32 cyclesToEndOfLine = 76 - ((mySystem->cycles() - 
+      (myClockWhenFrameStarted / 3)) % 76);
+
+  mySystem->incrementCycles(cyclesToEndOfLine-1);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::clearBuffers()
 {
@@ -1302,7 +1378,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
   updateFrame(clock + delay);
 
   // If a VSYNC hasn't been generated in time go ahead and end the frame
-  if(((clock - myClockWhenFrameStarted) / 228) > (Int32)myMaximumNumberOfScanlines)
+  if(((clock - myClockWhenFrameStarted) / 228) >= (Int32)myMaximumNumberOfScanlines)
   {
     mySystem->m6502().stop();
     myPartialFrameFlag = false;
@@ -1379,7 +1455,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
 
     case RSYNC:   // Reset horizontal sync counter
     {
-//      cerr << "TIA Poke: " << hex << addr << endl;
+      waitHorizontalRSync();
       break;
     }
 
@@ -1387,6 +1463,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
     {
       // TODO - 08-11-2009: determine correct delay instead of always
       //                    using '8' in TIATables::PokeDelay
+      updateFrame(clock + 8);
+
       myNUSIZ0 = value;
       mySuppressP0 = 0;
       break;
@@ -1396,6 +1474,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
     {
       // TODO - 08-11-2009: determine correct delay instead of always
       //                    using '8' in TIATables::PokeDelay
+      updateFrame(clock + 8);
+
       myNUSIZ1 = value;
       mySuppressP1 = 0;
       break;
@@ -1460,7 +1540,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       // Update the playfield mask based on reflection state if 
       // we're still on the left hand side of the playfield
       if(((clock - myClockWhenFrameStarted) % 228) < (68 + 79))
-        myCurrentPFMask = TIATables::PFMask[myCTRLPF & 0x01];
+        myPFMask = TIATables::PFMask[myCTRLPF & 0x01];
 
       break;
     }
@@ -1557,6 +1637,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       }
       if(myPOSP0 != newx)
       {
+        // TODO - update player timing
+
         // Find out under what condition the player is being reset
         delay = TIATables::PxPosResetWhen[myNUSIZ0 & 7][myPOSP0][newx];
 
@@ -1605,6 +1687,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       }
       if(myPOSP1 != newx)
       {
+        // TODO - update player timing
+
         // Find out under what condition the player is being reset
         delay = TIATables::PxPosResetWhen[myNUSIZ1 & 7][myPOSP1][newx];
 
@@ -1931,6 +2015,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         uInt16 middle = 4;
         switch(myNUSIZ0 & 0x07)
         {
+          // 1-pixel delay is taken care of in TIATables::PxMask
           case 0x05: middle = 8;  break;  // double size
           case 0x07: middle = 16; break;  // quad size
         }
@@ -1939,9 +2024,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         {
           myPOSM0 -= (8 - myMotionClockP0);
           myPOSM0 += (8 - myMotionClockM0);
-          if(myPOSM0 < 0)  myPOSM0 += 160;
         }
-        myPOSM0 %= 160;
+        CLAMP_POS(myPOSM0);
       }
       myRESMP0 = value & 0x02;
 
@@ -1960,6 +2044,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         uInt16 middle = 4;
         switch(myNUSIZ1 & 0x07)
         {
+          // 1-pixel delay is taken care of in TIATables::PxMask
           case 0x05: middle = 8;  break;  // double size
           case 0x07: middle = 16; break;  // quad size
         }
@@ -1968,9 +2053,8 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         {
           myPOSM1 -= (8 - myMotionClockP1);
           myPOSM1 += (8 - myMotionClockM1);
-          if(myPOSM1 < 0)  myPOSM1 += 160;
         }
-        myPOSM1 %= 160;
+        CLAMP_POS(myPOSM1);
       }
       myRESMP1 = value & 0x02;
 
@@ -1990,12 +2074,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       myHMOVEBlankEnabled = myAllowHMOVEBlanks ? 
         TIATables::HMOVEBlankEnableCycles[((clock - myClockWhenFrameStarted) % 228) / 3] : false;
 
-#ifdef USE_MMR_LATCHES
       // Do we have to undo some of the already applied cycles from an
       // active graphics latch?
       if(hpos + HBLANK < 17 * 4)
       {
-        Int16 cycle_fix = 17 - ((hpos + VBLANK + 7) / 4);
+        Int16 cycle_fix = 17 - ((hpos + HBLANK + 7) / 4);
         if(myHMP0mmr)  myPOSP0 = (myPOSP0 + cycle_fix) % 160;
         if(myHMP1mmr)  myPOSP1 = (myPOSP1 + cycle_fix) % 160;
         if(myHMM0mmr)  myPOSM0 = (myPOSM0 + cycle_fix) % 160;
@@ -2003,7 +2086,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         if(myHMBLmmr)  myPOSBL = (myPOSBL + cycle_fix) % 160;
       }
       myHMP0mmr = myHMP1mmr = myHMM0mmr = myHMM1mmr = myHMBLmmr = false;
-#endif
+
       // Can HMOVE activities be ignored?
       if(hpos >= -5 && hpos < 97 )
       {
@@ -2060,12 +2143,13 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       }
 
       // Make sure positions are in range
-      if(myPOSP0 < 0) { myPOSP0 += 160; }  myPOSP0 %= 160;
-      if(myPOSP1 < 0) { myPOSP1 += 160; }  myPOSP1 %= 160;
-      if(myPOSM0 < 0) { myPOSM0 += 160; }  myPOSM0 %= 160;
-      if(myPOSM1 < 0) { myPOSM1 += 160; }  myPOSM1 %= 160;
-      if(myPOSBL < 0) { myPOSBL += 160; }  myPOSBL %= 160;
+      CLAMP_POS(myPOSP0);
+      CLAMP_POS(myPOSP1);
+      CLAMP_POS(myPOSM0);
+      CLAMP_POS(myPOSM1);
+      CLAMP_POS(myPOSBL);
 
+      // TODO - handle late HMOVE's
       mySuppressP0 = mySuppressP1 = 0;
       break;
     }
@@ -2150,7 +2234,8 @@ void TIA::pokeHMP0(uInt8 value, Int32 clock)
       if(value != 0x70 && value != 0x80)
         myHMP0mmr = true;
     }
-    if(myPOSP0 < 0) { myPOSP0 += 160; }  myPOSP0 %= 160;
+    CLAMP_POS(myPOSP0);
+    // TODO - adjust player timing
   }
   myHMP0 = value;
 }
@@ -2183,7 +2268,8 @@ void TIA::pokeHMP1(uInt8 value, Int32 clock)
       if(value != 0x70 && value != 0x80)
         myHMP1mmr = true;
     }
-    if(myPOSP1 < 0) { myPOSP1 += 160; }  myPOSP1 %= 160;
+    CLAMP_POS(myPOSP1);
+    // TODO - adjust player timing
   }
   myHMP1 = value;
 }
@@ -2216,7 +2302,7 @@ void TIA::pokeHMM0(uInt8 value, Int32 clock)
       if(value != 0x70 && value != 0x80)
         myHMM0mmr = true;
     }
-    if(myPOSM0 < 0) { myPOSM0 += 160; }  myPOSM0 %= 160;
+    CLAMP_POS(myPOSM0);
   }
   myHMM0 = value;
 }
@@ -2249,7 +2335,7 @@ void TIA::pokeHMM1(uInt8 value, Int32 clock)
       if(value != 0x70 && value != 0x80)
         myHMM1mmr = true;
     }
-    if(myPOSM1 < 0) { myPOSM1 += 160; }  myPOSM1 %= 160;
+    CLAMP_POS(myPOSM1);
   }
   myHMM1 = value;
 }
@@ -2282,7 +2368,7 @@ void TIA::pokeHMBL(uInt8 value, Int32 clock)
       if(value != 0x70 && value != 0x80)
         myHMBLmmr = true;
     }
-    if(myPOSBL < 0) { myPOSBL += 160; }  myPOSBL %= 160;
+    CLAMP_POS(myPOSBL);
   }
   myHMBL = value;
 }
