@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GameInfoDialog.cxx 2529 2012-06-07 14:09:11Z stephena $
+// $Id: GameInfoDialog.cxx 2744 2013-05-29 16:45:18Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -53,7 +53,7 @@ GameInfoDialog::GameInfoDialog(
   const int vBorder = 4;
   int xpos, ypos, lwidth, fwidth, pwidth, tabID;
   WidgetArray wid;
-  StringMap items, ports, ctrls;
+  VariantList items, ports, ctrls;
 
   // Set real dimensions
   _w = 52 * fontWidth + 8;
@@ -70,7 +70,6 @@ GameInfoDialog::GameInfoDialog(
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos,
                         _h - buttonHeight - fontHeight - ifont.getLineHeight() - 20);
   addTabWidget(myTab);
-  addFocusWidget(myTab);
 
   // 1) Cartridge properties
   wid.clear();
@@ -136,7 +135,7 @@ GameInfoDialog::GameInfoDialog(
                        "Type:", kTextAlignLeft);
   pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   items.clear();
-  items.push_back("Auto-detect",           "AUTO-DETECT");
+  items.push_back("Auto-detect",                 "AUTO" );
   items.push_back("0840 (8K ECONObank)",         "0840" );
   items.push_back("2IN1 Multicart (4-32K)",      "2IN1" );
   items.push_back("4IN1 Multicart (8-32K)",      "4IN1" );
@@ -178,7 +177,7 @@ GameInfoDialog::GameInfoDialog(
   wid.push_back(myType);
 
   // Add items for tab 0
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
 
   // 2) Console properties
@@ -216,7 +215,7 @@ GameInfoDialog::GameInfoDialog(
   wid.push_back(myTVType);
 
   // Add items for tab 1
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
 
   // 3) Controller properties
@@ -326,7 +325,7 @@ GameInfoDialog::GameInfoDialog(
   wid.push_back(myMouseY);
 
   // Add items for tab 2
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
 
   // 4) Display properties
@@ -339,7 +338,7 @@ GameInfoDialog::GameInfoDialog(
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Format:", kTextAlignLeft);
   items.clear();
-  items.push_back("Auto-detect", "AUTO-DETECT");
+  items.push_back("Auto-detect", "AUTO");
   items.push_back("NTSC",    "NTSC");
   items.push_back("PAL",     "PAL");
   items.push_back("SECAM",   "SECAM");
@@ -389,7 +388,7 @@ GameInfoDialog::GameInfoDialog(
   myPPBlendLabel->setFlags(WIDGET_CLEARBG);
 
   // Add items for tab 3
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
 
   // Activate the first tab
@@ -448,14 +447,14 @@ void GameInfoDialog::loadView()
     return;
 
   // Cartridge properties
-  myName->setEditString(myGameProperties.get(Cartridge_Name));
+  myName->setText(myGameProperties.get(Cartridge_Name));
   myMD5->setLabel(myGameProperties.get(Cartridge_MD5));
-  myManufacturer->setEditString(myGameProperties.get(Cartridge_Manufacturer));
-  myModelNo->setEditString(myGameProperties.get(Cartridge_ModelNo));
-  myRarity->setEditString(myGameProperties.get(Cartridge_Rarity));
-  myNote->setEditString(myGameProperties.get(Cartridge_Note));
+  myManufacturer->setText(myGameProperties.get(Cartridge_Manufacturer));
+  myModelNo->setText(myGameProperties.get(Cartridge_ModelNo));
+  myRarity->setText(myGameProperties.get(Cartridge_Rarity));
+  myNote->setText(myGameProperties.get(Cartridge_Note));
   mySound->setSelected(myGameProperties.get(Cartridge_Sound), "MONO");
-  myType->setSelected(myGameProperties.get(Cartridge_Type), "AUTO-DETECT");
+  myType->setSelected(myGameProperties.get(Cartridge_Type), "AUTO");
 
   // Console properties
   myLeftDiff->setSelected(myGameProperties.get(Console_LeftDifficulty), "B");
@@ -488,9 +487,9 @@ void GameInfoDialog::loadView()
   myMouseY->setEnabled(!autoAxis);
 
   // Display properties
-  myFormat->setSelected(myGameProperties.get(Display_Format), "AUTO-DETECT");
-  myYStart->setEditString(myGameProperties.get(Display_YStart));
-  myHeight->setEditString(myGameProperties.get(Display_Height));
+  myFormat->setSelected(myGameProperties.get(Display_Format), "AUTO");
+  myYStart->setText(myGameProperties.get(Display_YStart));
+  myHeight->setText(myGameProperties.get(Display_Height));
 
   const string& phos = myGameProperties.get(Display_Phosphor);
   myPhosphor->setSelected(phos, "NO");
@@ -511,35 +510,36 @@ void GameInfoDialog::saveConfig()
     return;
 
   // Cartridge properties
-  myGameProperties.set(Cartridge_Name, myName->getEditString());
-  myGameProperties.set(Cartridge_Manufacturer, myManufacturer->getEditString());
-  myGameProperties.set(Cartridge_ModelNo, myModelNo->getEditString());
-  myGameProperties.set(Cartridge_Rarity, myRarity->getEditString());
-  myGameProperties.set(Cartridge_Note, myNote->getEditString());
-  myGameProperties.set(Cartridge_Sound, mySound->getSelectedTag());
-  myGameProperties.set(Cartridge_Type, myType->getSelectedTag());
+  myGameProperties.set(Cartridge_Name, myName->getText());
+  myGameProperties.set(Cartridge_Manufacturer, myManufacturer->getText());
+  myGameProperties.set(Cartridge_ModelNo, myModelNo->getText());
+  myGameProperties.set(Cartridge_Rarity, myRarity->getText());
+  myGameProperties.set(Cartridge_Note, myNote->getText());
+  myGameProperties.set(Cartridge_Sound, mySound->getSelectedTag().toString());
+  myGameProperties.set(Cartridge_Type, myType->getSelectedTag().toString());
 
   // Console properties
-  myGameProperties.set(Console_LeftDifficulty, myLeftDiff->getSelectedTag());
-  myGameProperties.set(Console_RightDifficulty, myRightDiff->getSelectedTag());
-  myGameProperties.set(Console_TelevisionType, myTVType->getSelectedTag());
+  myGameProperties.set(Console_LeftDifficulty, myLeftDiff->getSelectedTag().toString());
+  myGameProperties.set(Console_RightDifficulty, myRightDiff->getSelectedTag().toString());
+  myGameProperties.set(Console_TelevisionType, myTVType->getSelectedTag().toString());
 
   // Controller properties
-  myGameProperties.set(Controller_Left, myP0Controller->getSelectedTag());
-  myGameProperties.set(Controller_Right, myP1Controller->getSelectedTag());
+  myGameProperties.set(Controller_Left, myP0Controller->getSelectedTag().toString());
+  myGameProperties.set(Controller_Right, myP1Controller->getSelectedTag().toString());
   myGameProperties.set(Console_SwapPorts,
-    myLeftPort->getSelectedTag() == "L" ? "NO" : "YES");
-  myGameProperties.set(Controller_SwapPaddles, mySwapPaddles->getSelectedTag());
-  string mcontrol = myMouseControl->getSelectedTag();
+    myLeftPort->getSelectedTag().toString() == "L" ? "NO" : "YES");
+  myGameProperties.set(Controller_SwapPaddles, mySwapPaddles->getSelectedTag().toString());
+  string mcontrol = myMouseControl->getSelectedTag().toString();
   if(mcontrol != "auto")
-    mcontrol = myMouseX->getSelectedTag() + myMouseY->getSelectedTag();
+    mcontrol = myMouseX->getSelectedTag().toString() +
+               myMouseY->getSelectedTag().toString();
   myGameProperties.set(Controller_MouseAxis, mcontrol);
 
   // Display properties
-  myGameProperties.set(Display_Format, myFormat->getSelectedTag());
-  myGameProperties.set(Display_YStart, myYStart->getEditString());
-  myGameProperties.set(Display_Height, myHeight->getEditString());
-  myGameProperties.set(Display_Phosphor, myPhosphor->getSelectedTag());
+  myGameProperties.set(Display_Format, myFormat->getSelectedTag().toString());
+  myGameProperties.set(Display_YStart, myYStart->getText());
+  myGameProperties.set(Display_Height, myHeight->getText());
+  myGameProperties.set(Display_Phosphor, myPhosphor->getSelectedTag().toString());
   myGameProperties.set(Display_PPBlend, myPPBlendLabel->getLabel());
 
   // Determine whether to add or remove an entry from the properties set
@@ -593,7 +593,7 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kPhosphorChanged:
     {
-      bool status = myPhosphor->getSelectedTag() == "YES";
+      bool status = myPhosphor->getSelectedTag().toString() == "YES";
       myPPBlend->setEnabled(status);
       myPPBlendLabel->setEnabled(status);
       break;
@@ -605,7 +605,7 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kMCtrlChanged:
     {
-      bool state = myMouseControl->getSelectedTag() != "auto";
+      bool state = myMouseControl->getSelectedTag().toString() != "auto";
       myMouseX->setEnabled(state);
       myMouseY->setEnabled(state);
       break;
