@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GlobalPropsDialog.cxx 2530 2012-06-07 15:34:43Z stephena $
+// $Id: GlobalPropsDialog.cxx 2732 2013-05-09 14:22:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -46,7 +46,7 @@ GlobalPropsDialog::
   int lwidth = font.getStringWidth("Right Difficulty: "),
       pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   WidgetArray wid;
-  StringMap items;
+  VariantList items;
 
   // Set real dimensions
   _w = lwidth + pwidth + fontWidth*3 + 15;
@@ -64,7 +64,7 @@ GlobalPropsDialog::
   new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
                        "Bankswitch type:", kTextAlignLeft);
   items.clear();
-  items.push_back("Auto-detect",           "AUTO-DETECT");
+  items.push_back("Auto-detect",                 "AUTO" );
   items.push_back("0840 (8K ECONObank)",         "0840" );
   items.push_back("2IN1 Multicart (4-32K)",      "2IN1" );
   items.push_back("4IN1 Multicart (8-32K)",      "4IN1" );
@@ -184,7 +184,7 @@ void GlobalPropsDialog::loadConfig()
 {
   Settings& settings = instance().settings();
 
-  myBSType->setSelected(settings.getString("bs"), "AUTO-DETECT");
+  myBSType->setSelected(settings.getString("bs"), "AUTO");
   myLeftDiff->setSelected(settings.getString("ld"), "DEFAULT");
   myRightDiff->setSelected(settings.getString("rd"), "DEFAULT");
   myTVType->setSelected(settings.getString("tv"), "DEFAULT");
@@ -200,31 +200,31 @@ void GlobalPropsDialog::saveConfig()
   Settings& settings = instance().settings();
   string s;
 
-  s = myBSType->getSelectedTag();
-  if(s == "AUTO-DETECT") s = "";
-  settings.setString("bs", s);
+  s = myBSType->getSelectedTag().toString();
+  if(s == "AUTO") s = "";
+  settings.setValue("bs", s);
 
-  s = myLeftDiff->getSelectedTag();
+  s = myLeftDiff->getSelectedTag().toString();
   if(s == "DEFAULT") s = "";
-  settings.setString("ld", s);
+  settings.setValue("ld", s);
 
-  s = myRightDiff->getSelectedTag();
+  s = myRightDiff->getSelectedTag().toString();
   if(s == "DEFAULT") s = "";
-  settings.setString("rd", s);
+  settings.setValue("rd", s);
 
-  s = myTVType->getSelectedTag();
+  s = myTVType->getSelectedTag().toString();
   if(s == "DEFAULT") s = "";
-  settings.setString("tv", s);
+  settings.setValue("tv", s);
 
-  settings.setBool("holdselect", myHoldSelect->getState());
-  settings.setBool("holdreset", myHoldReset->getState());
-  settings.setBool("holdbutton0", myHoldButton0->getState());
+  settings.setValue("holdselect", myHoldSelect->getState());
+  settings.setValue("holdreset", myHoldReset->getState());
+  settings.setValue("holdbutton0", myHoldButton0->getState());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GlobalPropsDialog::setDefaults()
 {
-  myBSType->setSelected("AUTO-DETECT", "");
+  myBSType->setSelected("AUTO", "");
   myLeftDiff->setSelected("DEFAULT", "");
   myRightDiff->setSelected("DEFAULT", "");
   myTVType->setSelected("DEFAULT", "");
