@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: InputDialog.cxx 2444 2012-04-19 13:00:02Z stephena $
+// $Id: InputDialog.cxx 2744 2013-05-29 16:45:18Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -56,8 +56,6 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
   xpos = 2; ypos = vBorder;
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - buttonHeight - 20);
   addTabWidget(myTab);
-  wid.push_back(myTab);
-  addToFocusList(wid);
 
   // 1) Event mapper for emulation actions
   tabID = myTab->addTab("Emul. Events");
@@ -67,7 +65,7 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
                                              myTab->getHeight() - ypos,
                                              actions, kEmulationMode);
   myTab->setParentWidget(tabID, myEmulEventMapper);
-  addToFocusList(myEmulEventMapper->getFocusList(), tabID);
+  addToFocusList(myEmulEventMapper->getFocusList(), myTab, tabID);
 
   // 2) Event mapper for UI actions
   tabID = myTab->addTab("UI Events");
@@ -78,7 +76,7 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
                                              myTab->getHeight() - ypos,
                                              actions, kMenuMode);
   myTab->setParentWidget(tabID, myMenuEventMapper);
-  addToFocusList(myMenuEventMapper->getFocusList(), tabID);
+  addToFocusList(myMenuEventMapper->getFocusList(), myTab, tabID);
 
   // 3) Devices & ports
   addDevicePortTab(font);
@@ -110,7 +108,7 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
             fontHeight = font.getFontHeight();
   int xpos, ypos, lwidth, pwidth, tabID;
   WidgetArray wid;
-  StringMap items;
+  VariantList items;
 
   // Devices/ports
   tabID = myTab->addTab("Devices & Ports");
@@ -197,7 +195,7 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   wid.push_back(myMouseControl);
 
   // Add items for virtual device ports
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -221,7 +219,7 @@ void InputDialog::loadConfig()
   myMPaddleLabel->setLabel(instance().settings().getString("msense"));
 
   // AtariVox serial port
-  myAVoxPort->setEditString(instance().settings().getString("avoxport"));
+  myAVoxPort->setText(instance().settings().getString("avoxport"));
 
   // Allow all 4 joystick directions
   myAllowAll4->setState(instance().settings().getBool("joyallow4"));
@@ -236,36 +234,36 @@ void InputDialog::loadConfig()
 void InputDialog::saveConfig()
 {
   // Left & right ports
-  instance().eventHandler().mapStelladaptors(mySAPort->getSelectedTag());
+  instance().eventHandler().mapStelladaptors(mySAPort->getSelectedTag().toString());
 
   // Joystick deadzone
   int deadzone = myDeadzone->getValue();
-  instance().settings().setInt("joydeadzone", deadzone);
+  instance().settings().setValue("joydeadzone", deadzone);
   Joystick::setDeadZone(deadzone);
 
   // Grab mouse	 
-  instance().settings().setBool("grabmouse", myGrabMouse->getState());	 
+  instance().settings().setValue("grabmouse", myGrabMouse->getState());	 
   instance().frameBuffer().setCursorState();
 
   // Paddle speed (digital and mouse)
   int sensitivity = myDPaddleSpeed->getValue();
-  instance().settings().setInt("dsense", sensitivity);
+  instance().settings().setValue("dsense", sensitivity);
   Paddles::setDigitalSensitivity(sensitivity);
   sensitivity = myMPaddleSpeed->getValue();
-  instance().settings().setInt("msense", sensitivity);
+  instance().settings().setValue("msense", sensitivity);
   Paddles::setMouseSensitivity(sensitivity);
 
   // AtariVox serial port
-  instance().settings().setString("avoxport", myAVoxPort->getEditString());
+  instance().settings().setValue("avoxport", myAVoxPort->getText());
 
   // Allow all 4 joystick directions
   bool allowall4 = myAllowAll4->getState();
-  instance().settings().setBool("joyallow4", allowall4);
+  instance().settings().setValue("joyallow4", allowall4);
   instance().eventHandler().allowAllDirections(allowall4);
 
   // Use mouse as a controller
   bool usemouse = myMouseControl->getState();
-  instance().settings().setBool("usemouse", usemouse);
+  instance().settings().setValue("usemouse", usemouse);
   instance().eventHandler().setMouseControllerMode(usemouse);
 }
 
@@ -301,7 +299,7 @@ void InputDialog::setDefaults()
       myMPaddleLabel->setLabel("6");
 
       // AtariVox serial port
-      myAVoxPort->setEditString("");
+      myAVoxPort->setText("");
 
       // Allow all 4 joystick directions
       myAllowAll4->setState(false);
