@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherDialog.cxx 2530 2012-06-07 15:34:43Z stephena $
+// $Id: LauncherDialog.cxx 2751 2013-06-19 20:28:41Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -44,7 +44,6 @@
 #include "StringList.hxx"
 #include "StringListWidget.hxx"
 #include "Widget.hxx"
-#include "unzip.h"
 
 #include "LauncherDialog.hxx"
 
@@ -111,7 +110,6 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   int listWidth = _w - (romWidth > 0 ? romWidth+5 : 0) - 20;
   myList = new StringListWidget(this, font, xpos, ypos,
                                 listWidth, _h - 28 - bheight - 2*fontHeight);
-  myList->setNumberingMode(kListNumberingOff);
   myList->setEditable(false);
   wid.push_back(myList);
   if(myPattern)  wid.push_back(myPattern);  // Add after the list for tab order
@@ -176,7 +174,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   mySelectedItem = 0;  // Highlight 'Rom Listing'
 
   // Create an options dialog, similar to the in-game one
-  myOptions = new OptionsDialog(osystem, parent, this, w, h, true);  // not in game mode
+  myOptions = new OptionsDialog(osystem, parent, this, int(w * 0.8), int(h * 0.8), true);
 
   // Create a game list, which contains all the information about a ROM that
   // the launcher needs
@@ -185,7 +183,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   addToFocusList(wid);
 
   // Create context menu for ROM list options
-  StringMap l;
+  VariantList l;
   l.push_back("Override properties", "override");
   l.push_back("Filter listing", "filter");
   l.push_back("Reload listing", "reload");
@@ -216,18 +214,19 @@ LauncherDialog::~LauncherDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const string& LauncherDialog::selectedRomMD5()
 {
-  string extension;
   int item = myList->getSelected();
-  if(item < 0 || myGameList->isDir(item) ||
-     !LauncherFilterDialog::isValidRomName(myGameList->name(item), extension))
+  if(item < 0)
+    return EmptyString;
+
+  string extension;
+  const FilesystemNode node(myGameList->path(item));
+  if(node.isDirectory() || !LauncherFilterDialog::isValidRomName(node, extension))
     return EmptyString;
 
   // Make sure we have a valid md5 for this ROM
   if(myGameList->md5(item) == "")
-  {
-    const string& md5 = instance().MD5FromFile(myGameList->path(item));
-    myGameList->setMd5(item, md5);
-  }
+    myGameList->setMd5(item, MD5(node));
+
   return myGameList->md5(item);
 }
 
@@ -251,8 +250,8 @@ void LauncherDialog::loadConfig()
       msg.push_back("");
       msg.push_back("Click 'OK' to select a default ROM directory,");
       msg.push_back("or 'Cancel' to browse the filesystem manually.");
-      myFirstRunMsg = new MessageBox(this, instance().font(), msg,
-                                     _w, _h, kFirstRunMsgChosenCmd);
+      myFirstRunMsg = new GUI::MessageBox(this, instance().font(), msg,
+                                          _w, _h, kFirstRunMsgChosenCmd);
     }
     myFirstRunMsg->show();
   }
@@ -296,7 +295,7 @@ void LauncherDialog::updateListing(const string& nameToSelect)
   myPrevDirButton->setEnabled(myCurrentNode.hasParent());
 
   // Show current directory
-  myDir->setLabel(myCurrentNode.getRelativePath());
+  myDir->setLabel(myCurrentNode.getShortPath());
 
   // Now fill the list widget with the contents of the GameList
   StringList l;
@@ -311,108 +310,38 @@ void LauncherDialog::updateListing(const string& nameToSelect)
   myRomCount->setLabel(buf.str());
 
   // Restore last selection
-  int selected = -1;
-  if(!myList->getList().isEmpty())
-  {
-    const string& find =
-      nameToSelect == "" ? instance().settings().getString("lastrom") : nameToSelect;
-
-    if(find == "")
-      selected = 0;
-    else
-    {
-      unsigned int itemToSelect = 0;
-      StringList::const_iterator iter;
-      for(iter = myList->getList().begin(); iter != myList->getList().end();
-          ++iter, ++itemToSelect)	 
-      {
-        if(find == *iter)
-        {
-          selected = itemToSelect;
-          break;
-        }
-      }
-      if(itemToSelect > myList->getList().size())
-        selected = 0;
-    }
-  }
-  myList->setSelected(selected);
+  const string& find =
+    nameToSelect == "" ? instance().settings().getString("lastrom") : nameToSelect;
+  myList->setSelected(find);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::loadDirListing()
 {
-  if(!myCurrentNode.isDirectory() && !myCurrentNode.exists())
+  if(!myCurrentNode.isDirectory())
     return;
 
   FSList files;
   files.reserve(2048);
-
-  if(myCurrentNode.isDirectory())
-  {
-    myCurrentNode.getChildren(files, FilesystemNode::kListAll);
-  }
-  else
-  {
-    unzFile tz;
-    if((tz = unzOpen(myCurrentNode.getPath().c_str())) != NULL)
-    {
-      if(unzGoToFirstFile(tz) == UNZ_OK)
-      {
-        unz_file_info ufo;
-
-        for(;;)  // Loop through all files for valid 2600 images
-        {
-          // Longer filenames might be possible, but I don't
-          // think people would name files that long in zip files...
-          char filename[1024];
-
-          unzGetCurrentFileInfo(tz, &ufo, filename, 1024, 0, 0, 0, 0);
-          filename[1023] = '\0';
-
-          if(strlen(filename) >= 4 &&
-             !BSPF_startsWithIgnoreCase(filename, "__MACOSX"))
-          {
-            // Grab 3-character extension
-            const char* ext = filename + strlen(filename) - 4;
-
-            if(BSPF_equalsIgnoreCase(ext, ".a26") || BSPF_equalsIgnoreCase(ext, ".bin") ||
-               BSPF_equalsIgnoreCase(ext, ".rom"))
-            {
-              FilesystemNode newFile(AbstractFilesystemNode::getAbsolutePath(
-                  filename, myCurrentNode.getPath(), ""));
-              files.push_back(newFile);
-            }
-          }
-
-          // Scan the next file in the zip
-          if(unzGoToNextFile(tz) != UNZ_OK)
-            break;
-        }
-      }
-    }
-  }
+  myCurrentNode.getChildren(files, FilesystemNode::kListAll);
 
   // Add '[..]' to indicate previous folder
   if(myCurrentNode.hasParent())
     myGameList->appendGame(" [..]", "", "", true);
 
   // Now add the directory entries
-  bool domatch = myPattern && myPattern->getEditString() != "";
+  bool domatch = myPattern && myPattern->getText() != "";
   for(unsigned int idx = 0; idx < files.size(); idx++)
   {
-    string name = files[idx].getDisplayName();
     bool isDir = files[idx].isDirectory();
+    const string& name = isDir ? (" [" + files[idx].getName() + "]")
+                               : files[idx].getName();
 
     // Honour the filtering settings
     // Showing only certain ROM extensions is determined by the extension
     // that we want - if there are no extensions, it implies show all files
     // In this way, showing all files is on the 'fast code path'
-    if(isDir)
-    {
-      name = " [" + name + "]";
-    }
-    else if(myRomExts.size() > 0)
+    if(!isDir && myRomExts.size() > 0)
     {
       // Skip over those names we've filtered out
       if(!LauncherFilterDialog::isValidRomName(name, myRomExts))
@@ -420,7 +349,7 @@ void LauncherDialog::loadDirListing()
     }
 
     // Skip over files that don't match the pattern in the 'pattern' textbox
-    if(domatch && !isDir && !matchPattern(name, myPattern->getEditString()))
+    if(domatch && !isDir && !matchPattern(name, myPattern->getText()))
       continue;
 
     myGameList->appendGame(name, files[idx].getPath(), "", isDir);
@@ -438,16 +367,16 @@ void LauncherDialog::loadRomInfo()
   if(item < 0) return;
 
   string extension;
-  if(!myGameList->isDir(item) &&
-     LauncherFilterDialog::isValidRomName(myGameList->name(item), extension))
+  const FilesystemNode node(myGameList->path(item));
+  if(!node.isDirectory() && LauncherFilterDialog::isValidRomName(node, extension))
   {
     // Make sure we have a valid md5 for this ROM
     if(myGameList->md5(item) == "")
-      myGameList->setMd5(item, instance().MD5FromFile(myGameList->path(item)));
+      myGameList->setMd5(item, MD5(node));
 
     // Get the properties for this entry
     Properties props;
-    instance().propSet().getMD5(myGameList->md5(item), props);
+    instance().propSet().getMD5WithInsert(node, myGameList->md5(item), props);
 
     myRomInfoWidget->setProperties(props);
   }
@@ -458,15 +387,15 @@ void LauncherDialog::loadRomInfo()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::handleContextMenu()
 {
-  const string& cmd = myMenu->getSelectedTag();
+  const string& cmd = myMenu->getSelectedTag().toString();
 
   if(cmd == "override")
   {
-    parent().addDialog(myGlobalProps);
+    myGlobalProps->open();
   }
   else if(cmd == "filter")
   {
-    parent().addDialog(myFilters);
+    myFilters->open();
   }
   else if(cmd == "reload")
   {
@@ -525,47 +454,6 @@ bool LauncherDialog::matchPattern(const string& s, const string& pattern) const
   return false;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int LauncherDialog::filesInArchive(const string& archive)
-{
-  int count = -1;
-  unzFile tz;
-  if((tz = unzOpen(archive.c_str())) != NULL)
-  {
-    count = 0;
-    if(unzGoToFirstFile(tz) == UNZ_OK)
-    {
-      unz_file_info ufo;
-
-      for(;;)  // Loop through all files for valid 2600 images
-      {
-        // Longer filenames might be possible, but I don't
-        // think people would name files that long in zip files...
-        char filename[1024];
-
-        unzGetCurrentFileInfo(tz, &ufo, filename, 1024, 0, 0, 0, 0);
-        filename[1023] = '\0';
-
-        if(strlen(filename) >= 4 &&
-           !BSPF_startsWithIgnoreCase(filename, "__MACOSX"))
-        {
-          // Grab 3-character extension
-          const char* ext = filename + strlen(filename) - 4;
-
-          if(BSPF_equalsIgnoreCase(ext, ".a26") || BSPF_equalsIgnoreCase(ext, ".bin") ||
-             BSPF_equalsIgnoreCase(ext, ".rom"))
-            ++count;
-        }
-
-        // Scan the next file in the zip
-        if(unzGoToNextFile(tz) != UNZ_OK)
-          break;
-      }
-    }
-  }
-  return count;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod, char ascii)
 {
@@ -597,28 +485,16 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
   switch (cmd)
   {
     case kStartCmd:
-    case kListItemActivatedCmd:
-    case kListItemDoubleClickedCmd:
+    case ListWidget::kActivatedCmd:
+    case ListWidget::kDoubleClickedCmd:
     {
       int item = myList->getSelected();
       if(item >= 0)
       {
-        const string& rom = myGameList->path(item);
-        const string& md5 = myGameList->md5(item);
-        string extension;
-
-        int numFilesInArchive = filesInArchive(rom);
-        bool isArchive = !myGameList->isDir(item) &&
-                         BSPF_endsWithIgnoreCase(rom, ".zip");
+        const FilesystemNode romnode(myGameList->path(item));
 
         // Directory's should be selected (ie, enter them and redisplay)
-        // Archives should be entered if they contain more than 1 file
-        if(isArchive && numFilesInArchive < 1)
-        {
-          instance().frameBuffer().showMessage("Archive does not contain any valid ROM files",
-                                               kMiddleCenter, true);
-        }
-        else if((isArchive && numFilesInArchive > 1) || myGameList->isDir(item))
+        if(romnode.isDirectory())
         {
           string dirname = "";
           if(myGameList->name(item) == " [..]")
@@ -629,41 +505,35 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
           }
           else
           {
-            myCurrentNode = FilesystemNode(rom);
+            myCurrentNode = romnode;
             myNodeNames.push(myGameList->name(item));
           }
           updateListing(dirname);
         }
         else
         {
-          if(LauncherFilterDialog::isValidRomName(rom, extension))
-          {
-            if(instance().createConsole(rom, md5))
-              instance().settings().setString("lastrom", myList->getSelectedString());
-            else
-              instance().frameBuffer().showMessage(
-                  "Error creating console (check ROM file/bankswitch scheme)",
-                  kMiddleCenter, true);
-          }
+          const string& result =
+            instance().createConsole(romnode, myGameList->md5(item));
+          if(result == EmptyString)
+            instance().settings().setValue("lastrom", myList->getSelectedString());
           else
-            instance().frameBuffer().showMessage("Not a valid ROM file",
-                                                 kMiddleCenter, true);
+            instance().frameBuffer().showMessage(result, kMiddleCenter, true);
         }
       }
       break;
     }
 
     case kOptionsCmd:
-      parent().addDialog(myOptions);
+      myOptions->open();
       break;
 
     case kPrevDirCmd:
-    case kListPrevDirCmd:
+    case ListWidget::kPrevDirCmd:
       myCurrentNode = myCurrentNode.getParent();
       updateListing(myNodeNames.empty() ? "" : myNodeNames.pop());
       break;
 
-    case kListSelectionChangedCmd:
+    case ListWidget::kSelectionChangedCmd:
       loadRomInfo();
       break;
 
@@ -678,13 +548,13 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
         myRomDir = new BrowserDialog(this, instance().font(), _w, _h);
 
       myRomDir->show("Select ROM directory:", "~",
-                     FilesystemNode::kListDirectoriesOnly, kStartupRomDirChosenCmd);
+                     BrowserDialog::Directories, kStartupRomDirChosenCmd);
       break;
 
     case kStartupRomDirChosenCmd:
     {
       FilesystemNode dir(myRomDir->getResult());
-      instance().settings().setString("romdir", dir.getRelativePath());
+      instance().settings().setValue("romdir", dir.getShortPath());
       // fall through to the next case
     }
     case kRomDirChosenCmd:
@@ -694,10 +564,6 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       updateListing();
       break;
 
-    case kSnapDirChosenCmd:
-      // Stub just in case we need it
-      break;
-
     case kReloadRomDirCmd:
       updateListing();
       break;
@@ -707,12 +573,12 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       updateListing();
       break;
 
-    case kCMenuItemSelectedCmd:
+    case ContextMenu::kItemSelectedCmd:
       handleContextMenu();
       break;
 
-    case kEditAcceptCmd:
-    case kEditChangedCmd:
+    case EditableWidget::kAcceptCmd:
+    case EditableWidget::kChangedCmd:
       // The updateListing() method knows what to do when the text changes
       updateListing();
       break;
