@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherFilterDialog.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: LauncherFilterDialog.cxx 2751 2013-06-19 20:28:41Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -48,7 +48,7 @@ LauncherFilterDialog::LauncherFilterDialog(GuiObject* boss, const GUI::Font& fon
   int lwidth = font.getStringWidth("Show: "),
       pwidth = font.getStringWidth("ROMs ending with");
   WidgetArray wid;
-  StringMap items;
+  VariantList items;
 
   // Set real dimensions
   _w = 3 * buttonWidth;//lwidth + pwidth + fontWidth*5 + 10;
@@ -144,8 +144,9 @@ bool LauncherFilterDialog::isValidRomName(const string& name,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool LauncherFilterDialog::isValidRomName(const string& name, string& ext)
+bool LauncherFilterDialog::isValidRomName(const FilesystemNode& node, string& ext)
 {
+  const string& name = node.getPath();
   string::size_type idx = name.find_last_of('.');
   if(idx != string::npos)
   {
@@ -172,9 +173,9 @@ void LauncherFilterDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherFilterDialog::saveConfig()
 {
-  const string& type = myFileType->getSelectedTag();
+  const string& type = myFileType->getSelectedTag().toString();
   if(type == "allfiles" || type == "allroms")
-    instance().settings().setString("launcherexts", type);
+    instance().settings().setValue("launcherexts", type);
   else
   {
     ostringstream buf;
@@ -184,13 +185,13 @@ void LauncherFilterDialog::saveConfig()
 
     // No ROMs selected means use all files
     if(buf.str() == "")
-      instance().settings().setString("launcherexts", "allfiles");
+      instance().settings().setValue("launcherexts", "allfiles");
     else
-      instance().settings().setString("launcherexts", buf.str());
+      instance().settings().setValue("launcherexts", buf.str());
   }
 
   // Let parent know about the changes
-  sendCommand(kReloadFiltersCmd, 0, 0);
+  sendCommand(LauncherDialog::kReloadFiltersCmd, 0, 0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -250,7 +251,7 @@ void LauncherFilterDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kFileTypeChanged:
-      handleFileTypeChange(myFileType->getSelectedTag());
+      handleFileTypeChange(myFileType->getSelectedTag().toString());
       break;
 
     default:
