@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: MessageBox.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: MessageBox.cxx 2601 2013-02-10 22:26:06Z stephena $
 //============================================================================
 
 #include "Dialog.hxx"
@@ -25,6 +25,8 @@
 
 #include "MessageBox.hxx"
 
+namespace GUI {
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MessageBox::MessageBox(GuiObject* boss, const GUI::Font& font,
                        const StringList& text, int max_w, int max_h, int cmd,
@@ -107,3 +109,5 @@ void MessageBox::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
   }
 }
+
+}  // namespace GUI
