@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PopUpWidget.cxx 2438 2012-04-11 16:10:24Z stephena $
+// $Id: PopUpWidget.cxx 2732 2013-05-09 14:22:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -44,7 +44,7 @@ static unsigned int up_down_arrows[8] = {
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
-                         int x, int y, int w, int h, const StringMap& list,
+                         int x, int y, int w, int h, const VariantList& list,
                          const string& label, int labelWidth, int cmd)
   : Widget(boss, font, x, y - 1, w, h + 2),
     CommandSender(boss),
@@ -91,6 +91,18 @@ void PopUpWidget::handleMouseDown(int x, int y, int button, int clickCount)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::handleMouseWheel(int x, int y, int direction)
+{
+  if(isEnabled() && !myMenu->isVisible())
+  {
+    if(direction < 0)
+      myMenu->sendSelectionUp();
+    else
+      myMenu->sendSelectionDown();
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool PopUpWidget::handleEvent(Event::Type e)
 {
@@ -102,6 +114,18 @@ bool PopUpWidget::handleEvent(Event::Type e)
     case Event::UISelect:
       handleMouseDown(0, 0, 1, 0);
       return true;
+    case Event::UIUp:
+    case Event::UILeft:
+    case Event::UIPgUp:
+      return myMenu->sendSelectionUp();
+    case Event::UIDown:
+    case Event::UIRight:
+    case Event::UIPgDown:
+      return myMenu->sendSelectionDown();
+    case Event::UIHome:
+      return myMenu->sendSelectionFirst();
+    case Event::UIEnd:
+      return myMenu->sendSelectionLast();
     default:
       return false;
   }
