@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: StringListWidget.cxx 2438 2012-04-11 16:10:24Z stephena $
+// $Id: StringListWidget.cxx 2719 2013-05-06 15:32:03Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -27,8 +27,9 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 StringListWidget::StringListWidget(GuiObject* boss, const GUI::Font& font,
-                                   int x, int y, int w, int h)
-  : ListWidget(boss, font, x, y, w, h)
+                                   int x, int y, int w, int h, bool hilite)
+  : ListWidget(boss, font, x, y, w, h),
+    _hilite(hilite)
 {
 }
 
@@ -62,12 +63,10 @@ void StringListWidget::drawWidget(bool hilite)
   // Draw the list items
   for (i = 0, pos = _currentPos; i < _rows && pos < len; i++, pos++)
   {
-    const uInt32 textColor = (_selectedItem == pos && _editMode)
-                              ? kColor : kTextColor;
     const int y = _y + 2 + _fontHeight * i;
 
     // Draw the selected item inverted, on a highlighted background.
-    if (_selectedItem == pos)
+    if (_selectedItem == pos && _hilite)
     {
       if (_hasFocus && !_editMode)
         s.fillRect(_x + 1, _y + 1 + _fontHeight * i, _w - 1, _fontHeight, kTextColorHi);
@@ -75,15 +74,6 @@ void StringListWidget::drawWidget(bool hilite)
         s.frameRect(_x + 1, _y + 1 + _fontHeight * i, _w - 1, _fontHeight, kTextColorHi);
     }
 
-    // If in numbering mode, we first print a number prefix
-    if (_numberingMode != kListNumberingOff)
-    {
-      char temp[10];
-      BSPF_snprintf(temp, 9, "%2d. ", (pos + _numberingMode));
-      buffer = temp;
-      s.drawString(_font, buffer, _x + 2, y, _w - 4, textColor);
-    }
-
     GUI::Rect r(getEditRect());
     if (_selectedItem == pos && _editMode)
     {
@@ -111,18 +101,10 @@ void StringListWidget::drawWidget(bool hilite)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 GUI::Rect StringListWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w - 2 , _fontHeight);
+  GUI::Rect r(2, 1, _w - 2, _fontHeight);
   const int offset = (_selectedItem - _currentPos) * _fontHeight;
   r.top += offset;
   r.bottom += offset;
-
-  if (_numberingMode != kListNumberingOff)
-  {
-    char temp[10];
-    // FIXME: Assumes that all digits have the same width.
-    BSPF_snprintf(temp, 9, "%2d. ", (_list.size() - 1 + _numberingMode));
-    r.left += _font.getStringWidth(temp);
-  }
 	
   return r;
 }
