@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: UIDialog.cxx 2448 2012-04-20 20:47:53Z stephena $
+// $Id: UIDialog.cxx 2732 2013-05-09 14:22:34Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -51,7 +51,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   int xpos, ypos, tabID;
   int lwidth, pwidth = font.getStringWidth("Standard");
   WidgetArray wid;
-  StringMap items;
+  VariantList items;
 
   // Set real dimensions
   _w = 37 * fontWidth + 10;
@@ -61,7 +61,6 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   xpos = ypos = vBorder;
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - buttonHeight - 20);
   addTabWidget(myTab);
-  addFocusWidget(myTab);
 
   //////////////////////////////////////////////////////////
   // 1) Launcher options
@@ -156,7 +155,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
                        kTextAlignLeft);
 
   // Add items for tab 0
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
   //////////////////////////////////////////////////////////
   // 2) Debugger options
@@ -170,7 +169,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   myDebuggerWidthSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
                                            lineHeight, "Debugger Width: ",
                                            lwidth, kDWidthChanged);
-  myDebuggerWidthSlider->setMinValue(1050);
+  myDebuggerWidthSlider->setMinValue(1080);
   myDebuggerWidthSlider->setMaxValue(1920);
   myDebuggerWidthSlider->setStepValue(10);
   wid.push_back(myDebuggerWidthSlider);
@@ -184,7 +183,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   myDebuggerHeightSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
                                             lineHeight, "Debugger Height: ",
                                             lwidth, kDHeightChanged);
-  myDebuggerHeightSlider->setMinValue(700);
+  myDebuggerHeightSlider->setMinValue(720);
   myDebuggerHeightSlider->setMaxValue(1200);
   myDebuggerHeightSlider->setStepValue(10);
   wid.push_back(myDebuggerHeightSlider);
@@ -211,7 +210,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   }
 
   // Add items for tab 1
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
   //////////////////////////////////////////////////////////
   // 3) Misc. options
@@ -263,7 +262,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   ypos += lineHeight + 4;
 
   // Add items for tab 2
-  addToFocusList(wid, tabID);
+  addToFocusList(wid, myTab, tabID);
 
   // Activate the first tab
   myTab->setActiveTab(0);
@@ -286,10 +285,10 @@ UIDialog::~UIDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void UIDialog::loadConfig()
 {
-  int w, h;
-
   // Launcher size
-  instance().settings().getSize("launcherres", w, h);
+  const GUI::Size& ls = instance().settings().getSize("launcherres");
+  int w = ls.w, h = ls.h;
+
   w = BSPF_max(w, 320);
   h = BSPF_max(h, 240);
   w = BSPF_min(w, 1920);
@@ -314,9 +313,10 @@ void UIDialog::loadConfig()
 
 #ifdef DEBUGGER_SUPPORT
   // Debugger size
-  instance().settings().getSize("debuggerres", w, h);
-  w = BSPF_max(w, 1050);
-  h = BSPF_max(h, 620);
+  const GUI::Size& ds = instance().settings().getSize("debuggerres");
+  w = ds.w, h = ds.h;
+  w = BSPF_max(w, 1080);
+  h = BSPF_max(h, 720);
   w = BSPF_min(w, 1920);
   h = BSPF_min(h, 1200);
 
@@ -345,38 +345,40 @@ void UIDialog::loadConfig()
 void UIDialog::saveConfig()
 {
   // Launcher size
-  instance().settings().setSize("launcherres", 
-    myLauncherWidthSlider->getValue(), myLauncherHeightSlider->getValue());
+  instance().settings().setValue("launcherres", 
+    GUI::Size(myLauncherWidthSlider->getValue(),
+              myLauncherHeightSlider->getValue()));
 
   // Launcher font
-  instance().settings().setString("launcherfont",
-    myLauncherFontPopup->getSelectedTag());
+  instance().settings().setValue("launcherfont",
+    myLauncherFontPopup->getSelectedTag().toString());
 
   // ROM launcher info viewer
-  instance().settings().setString("romviewer",
-    myRomViewerPopup->getSelectedTag());
+  instance().settings().setValue("romviewer",
+    myRomViewerPopup->getSelectedTag().toString());
 
   // Exit to Launcher
-  instance().settings().setString("exitlauncher",
-    myLauncherExitPopup->getSelectedTag());
+  instance().settings().setValue("exitlauncher",
+    myLauncherExitPopup->getSelectedTag().toString());
 
   // Debugger size
-  instance().settings().setSize("debuggerres", 
-    myDebuggerWidthSlider->getValue(), myDebuggerHeightSlider->getValue());
+  instance().settings().setValue("debuggerres",
+    GUI::Size(myDebuggerWidthSlider->getValue(),
+              myDebuggerHeightSlider->getValue()));
 
   // UI palette
-  instance().settings().setString("uipalette",
-    myPalettePopup->getSelectedTag());
+  instance().settings().setValue("uipalette",
+    myPalettePopup->getSelectedTag().toString());
 
   // Listwidget quick delay
-  instance().settings().setString("listdelay",
-    myListDelayPopup->getSelectedTag());
-  ListWidget::setQuickSelectDelay(atoi(myListDelayPopup->getSelectedTag().c_str()));
+  instance().settings().setValue("listdelay",
+    myListDelayPopup->getSelectedTag().toString());
+  ListWidget::setQuickSelectDelay(myListDelayPopup->getSelectedTag().toInt());
 
   // Mouse wheel lines
-  instance().settings().setString("mwheel",
-    myWheelLinesPopup->getSelectedTag());
-  ScrollBarWidget::setWheelLines(atoi(myWheelLinesPopup->getSelectedTag().c_str()));
+  instance().settings().setValue("mwheel",
+    myWheelLinesPopup->getSelectedTag().toString());
+  ScrollBarWidget::setWheelLines(myWheelLinesPopup->getSelectedTag().toInt());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -399,10 +401,10 @@ void UIDialog::setDefaults()
     }
 
     case 1:  // Debugger options
-      myDebuggerWidthSlider->setValue(1050);
-      myDebuggerWidthLabel->setValue(1050);
-      myDebuggerHeightSlider->setValue(700);
-      myDebuggerHeightLabel->setValue(700);
+      myDebuggerWidthSlider->setValue(1080);
+      myDebuggerWidthLabel->setValue(1080);
+      myDebuggerHeightSlider->setValue(720);
+      myDebuggerHeightLabel->setValue(720);
       break;
 
     case 2:  // Misc. options
