@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystemMACOSX.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: OSystemMACOSX.cxx 2612 2013-02-16 19:56:09Z stephena $
 //============================================================================
 
 #include "FSNode.hxx"
@@ -46,8 +46,14 @@ OSystemMACOSX::~OSystemMACOSX()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string OSystemMACOSX::defaultSnapDir()
+string OSystemMACOSX::defaultSnapSaveDir()
 {
   FilesystemNode desktop("~/Desktop");
-  return desktop.isDirectory() ? desktop.getRelativePath() : "~";
+  return desktop.isDirectory() ? desktop.getShortPath() : "~";
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string OSystemMACOSX::defaultSnapLoadDir()
+{
+  return defaultSnapSaveDir();
 }
