@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SerialPortMACOSX.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: SerialPortMACOSX.cxx 2678 2013-03-31 00:10:05Z stephena $
 //============================================================================
 
 #include <stdio.h>
@@ -39,6 +39,7 @@ SerialPortMACOSX::SerialPortMACOSX()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SerialPortMACOSX::~SerialPortMACOSX()
 {
+  closePort();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -65,7 +66,10 @@ bool SerialPortMACOSX::openPort(const string& device)
 void SerialPortMACOSX::closePort()
 {
   if(myHandle)
+  {
     close(myHandle);
+    myHandle = 0;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
