@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2013 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsMACOSX.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: SettingsMACOSX.cxx 2728 2013-05-08 23:59:19Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -63,7 +63,7 @@ void SettingsMACOSX::saveConfig()
   // Write out each of the key and value pairs
   const SettingsArray& settings = getInternalSettings();
   for(unsigned int i = 0; i < settings.size(); ++i)
-    prefsSetString(settings[i].key.c_str(), settings[i].value.c_str());
+    prefsSetString(settings[i].key.c_str(), settings[i].value.toCString());
 
   prefsSave();
 }
